/*
** Module   : AclDef.java
** Abstract : Container for the ACL definitions
**
** Copyright (c) 2009-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description----------------------
** 001 NVS 20090616   @42713 Created initial version.
** 002 NVS 20090617   @42726 Simplified the ACL presentation into a flat form
**                           that can be presented as a table.
** 003 NVS 20090622   @42778 Added ACL identifier field
** 004 NVS 20090623   @42793 ACL identifiers are exposed as integer values.
** 005 NVS 20090629   @42996 Rights are passed as true instances. Implemented
**                           error tracking flags. left TO-DO marks for the
**                           Externalizable interface.
** 006 SBI 20170225          Moved Acl to its own module.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin;

import java.io.*;
import java.util.*;

/**
 * Defines entities that form an Access Control List in P2j environent.
 */
public class AclDef
implements Serializable
{
   /** Constant to specify ALL_OTHERS subject */
   public static final int ALL_OTHERS = -2;
   
   /** Names the resource this ACL is for */
   public String resourceName;
   
   /** 
    * Server name this ACL definition is for.
    * <p>
    * If not null, all ACLs belong to the same server's instance or 
    * shared instance (if empty string). The ACLs inside will have the server's
    * names as null.
    * <p>
    * If null, the array of ACLs is i combination and every ACL should have
    * its origin defined.
    */
   public String server;
   
   /**
    * Serves as a dictionary of account names that maps them into integers.
    * Those integers are used as references to the accounts in all data items
    * of this class.  
    */
   public Map<Integer, String> subjects = new HashMap<>();
   
   /** 
    * Array of ACLs. It comes from the server sorted by the oid, even if it 
    * describes a combined ACLs, This is the order the ACLs are used for
    * decision making.
    */
   public Acl[] acls;
   
   /** 
    * Error count. Reports the number of ACLs in the array having one or more
    * error flags on. 
    */
   public int errors;
}
