/*
** Module   : PrintingContext.java
** Abstract : PrintingContext defines methods that should be implemented by a view if it provides
**            print preview functionality.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170614 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/


package com.goldencode.p2j.admin.client.application;

import java.util.EnumMap;

import com.goldencode.p2j.admin.client.widget.EnumCreator;
import com.goldencode.p2j.admin.shared.ReportParameters;


/**
 * PrintingContext defines methods that should be implemented by a view if it provides print
 * preview functionality.
 */
public interface PrintingContext
{
   enum Range
   implements EnumCreator<Range>
   {
      /** The range given by the user's selection */
      SELECTION,
      
      /** The unbounded range */
      ALL;

      /**
       * Lists all values.
       * 
       * @return   The list of enumeration values.
       */
      @Override
      public Range[] getValues()
      {
         return values();
      }

      /**
       * Create a enumeration by its name.
       *  
       * @param    name
       *           The name
       * @return   The enumeration value.
       */
      @Override
      public Range createFrom(String name)
      {
         return valueOf(name);
      }
   }
   
   /**
    * Preview dialog parameters
    */
   enum PreviewDialogParameters
   {
      /** The title parameter key */
      TITLE,
      
      /** The help text parameter key */
      HELP_TEXT,
      
      /** The label parameter key for the ALL range option */
      RANGE_ALL_LABEL,
      
      /** The label parameter key for the SELECTION range option */
      RANGE_SELECTION_LABEL
   }
   
   /**
    * Returns pairs of a report parameter key and its value
    * 
    * @return   The enumeration map of report parameters keys to their values
    */
   EnumMap<ReportParameters, Object> getReportParameters();
   
   /**
    * Returns the size of all data to preview or the size of selected data to preview
    * 
    * @param    range
    *           The target range of selected data to preview
    * @return   The size of the selected data
    */
   int getDataSize(Range range);
   
   /**
    * Returns the value of the preview dialog parameter.
    * 
    * @param    parameter
    *           The preview dialog parameter key
    * 
    * @return   The value of the preview dialog parameter
    */
   String getPreviewDialogParameter(PreviewDialogParameters parameter);
   
   /**
    * Returns url path to the owner of this print preview view.
    * 
    * @return   The url path.
    */
   String getPathToPrintPreviewOwner();
}
