/*
** Module   : AccountsConstants.java
** Abstract : AccountsConstants holds constant strings used by accounts package.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client.application.home.accounts;

import java.util.Map;

import com.google.gwt.i18n.client.ConstantsWithLookup;
import com.google.gwt.i18n.client.Constants.DefaultStringValue;


/**
 * GWT constants lookup interface to be used to provide accounts constants.
 */
public interface AccountsConstants extends ConstantsWithLookup
{
   /**
    * Lists all authentication modes and their presentation values.
    * 
    * @return   The all authentication modes and their presentation values
    */
   @DefaultStringMapValue({ "SERVER_AUTH", "Use Server Auth Mode", "GROUP_SERVER_AUTH",
            "Use Group or Server Auth Mode", "PASSWORD_AUTH", "Password Authentication",
            "CERT_AUTH", "Certificate Authentication", "CERT_PASSWORD_AUTH",
            "Certificate + Password Authentication", "CUST_AUTH", "Custom Authentication" })
   Map<String, String> getAuthModes();

   /**
    * Returns the presentation value for the "Use Server Auth Mode" authentication mode.
    * 
    * @return   Use Server Auth Mode
    */
   @DefaultStringValue("Use Server Auth Mode")
   String getUseServerAuth();

   /**
    * Returns the presentation value for the "Use Group or Server Auth Mode" authentication mode.
    * 
    * @return   Use Group or Server Auth Mode
    */
   @DefaultStringValue("Use Group or Server Auth Mode")
   String getGroupAndServerAuth();

   /**
    * Returns the presentation value for the "Password Authentication" authentication mode.
    * 
    * @return   Password Authentication
    */
   @DefaultStringValue("Password Authentication")
   String getPasswordAuth();

   /**
    * Returns the presentation value for the "Certificate Authentication" authentication mode.
    * 
    * @return   Certificate Authentication
    */
   @DefaultStringValue("Certificate Authentication")
   String getCertificateAuth();

   /**
    * Returns the presentation value for the "Certificate + Password Authentication"
    * authentication mode.
    * 
    * @return   Certificate + Password Authentication
    */
   @DefaultStringValue("Certificate + Password Authentication")
   String getCertificateAndPasswordAuth();

   /**
    * Returns the presentation value for the "Custom Authentication" authentication mode.
    * 
    * @return   Custom Authentication
    */
   @DefaultStringValue("Custom Authentication")
   String getCustomAuth();

   /**
    * Lists all security options and their presentation values.
    * 
    * @return   The all security options and their presentation values
    */
   @DefaultStringMapValue({ "LEAVE_PASSWORD_UNCHANGED", "Password Unchanged",
            "ENTER_PASSWORD", "Enter Password", "NO_PASSWORD_PROTECTION",
            "No Password Protection" })
   Map<String, String> getSecuritySelectorOptions();

   /**
    * Returns the presentation value for the "Password Unchanged" security option.
    * 
    * @return   Password Unchanged
    */
   @DefaultStringValue("Password Unchanged")
   String getLeavePasswordUnchanged();

   /**
    * Returns the presentation value for the "Enter Password" security option.
    * 
    * @return   Enter Password
    */
   @DefaultStringValue("Enter Password")
   String getEnterPassword();

   /**
    * Returns the presentation value for the "No Password Protection" security option.
    * 
    * @return   No Password Protection
    */
   @DefaultStringValue("No Password")
   String getNoPasswordProtection();

   /**
    * Lists all password options and their presentation values.
    * 
    * @return   The all password options and their presentation values
    */
   @DefaultStringMapValue({ "CLONE_PASSWORD", "Clone Password", "NEW_PASSWORD",
            "Input New Password" })
   Map<String, String> getPasswordOptions();

   /**
    * Lists all certificate options and their presentation values.
    * 
    * @return   The all certificate options and their presentation values
    */
   @DefaultStringMapValue({ "CA", "CA Certificates", "PEER", "Peer certificates" })
   Map<String, String> getCertificatesOptions();

   /**
    * Lists all account targets and their presentation values.
    * 
    * @return   The all account targets and their presentation values
    */
   @DefaultStringMapValue({ "GROUP", "All Group Accounts", "USER",
      "All User Accounts (except the one you are using now)" })
   Map<String, String> getAccountsTargets();

}
