/*
** Module   : PasswordOptions.java
** Abstract : PasswordOptions represents a password's options.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client.application.home.accounts;

import java.util.Map;

import com.goldencode.p2j.admin.client.widget.EnumCreator;
import com.google.gwt.core.shared.GWT;

/**
 * PasswordOptions represents a password's options.
 */
public enum PasswordOptions implements EnumCreator<PasswordOptions>
{
   /** Defines the clone password option */
   CLONE_PASSWORD,
   
   /** Defines the new password option */
   NEW_PASSWORD;

   /** The instance of accounts constants provider */
   private static final AccountsConstants CONSTANTS = GWT.create(AccountsConstants.class);
   
   /** The password options and their presentation values */
   private static final Map<String, String> MAP = CONSTANTS.getPasswordOptions();
   
   /**
    * Lists all its values.
    * 
    * @return   The list of enumeration values.
    */
   @Override
   public PasswordOptions[] getValues()
   {
      return values();
   }

   /**
    * Create this enumeration by its name.
    *  
    * @param    name
    *           The password option name
    * @return   The password option enumeration value.
    */
   @Override
   public PasswordOptions createFrom(String name)
   {
      return valueOf(name);
   }
   
   /**
    * Returns its presentation value.
    * 
    * @return   The presentation value for this password option
    */
   @Override
   public String toString()
   {
      return MAP.get(name());
   }
}
