/*
** Module   : CertificateDefinitionView.java
** Abstract : CertificateDefinitionView represents a PEM Certificate Upload form view.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client.application.home.accounts.certificates;

import java.util.logging.Level;

import org.gwtbootstrap3.client.ui.Button;
import org.gwtbootstrap3.client.ui.Modal;
import org.gwtbootstrap3.client.ui.TextArea;
import org.gwtbootstrap3.client.ui.TextBox;

import com.goldencode.p2j.admin.client.application.home.accounts.CertificatesOptions;
import com.goldencode.p2j.admin.client.widget.dialog.ModalViewWithUiHandlers;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiHandler;
import com.google.gwt.user.client.ui.FileUpload;
import com.google.gwt.user.client.ui.Widget;
import com.google.inject.Inject;

/**
 * Certificate Definition View is a modal dialog that provides UI to upload new certificate from
 * the client to the server. PEM certificates are only supported. 
 */
public class CertificateDefinitionView
extends ModalViewWithUiHandlers<CertificatesUIHandlers>
{
   /**
    * GWT UI creator
    */
   interface Binder
   extends UiBinder<Widget, CertificateDefinitionView>
   {}
   
   /** Underlined modal dialog */
   @UiField
   Modal modal;
   
   /** Represents a certificate alias */
   @UiField
   TextBox alias;
   
   /** The file upload widget to load PEM certificate */
   @UiField
   FileUpload importPEMCert;
   
   /** Displays a PEM certificate */
   @UiField
   TextArea pem;
   
   /** The button to save certificate changes */
   @UiField
   Button saveAction;
   
   /** The button to close this dialog and to cancel changes */
   @UiField
   Button cancelAction;
   
   /** The certificate role of the uploaded certificate */
   private CertificatesOptions certificateOption;
   
   /**
    * Builds a certificate view by injecting GWT binder as its parameter
    * 
    * @param    binder
    *           GWT UI Binder
    */
   @Inject
   public CertificateDefinitionView(CertificateDefinitionView.Binder binder)
   {
      initWidget(binder.createAndBindUi(this));
      pem.setReadOnly(true);
      addShowHandler(() -> {
         resetField(importPEMCert.getElement());
         resetField(pem.getElement());
      });
      importPEMCert.addChangeHandler(new ChangeHandler()
      {
         @Override
         public void onChange(ChangeEvent event)
         {
            readFile(event.getNativeEvent(), pem.getElement());
         }
      });
   }
   
   /**
    * Returns the selected certificate role.
    * 
    * @return   The certificate role.
    */
   public CertificatesOptions getCertificateOption()
   {
      return certificateOption;
   }

   /**
    * Provides a certificate role.
    * 
    * @param    certificateOption
    *           The certificate role.
    */
   public void setCertificateOption(CertificatesOptions certificateOption)
   {
      this.certificateOption = certificateOption;
   }

   /**
    * A click button handler attached to the "Save" button.
    * 
    * @param    e
    *           Click event
    */
   @UiHandler("saveAction")
   void onSaveAction(ClickEvent e)
   {
      logger.log(Level.INFO, "onSaveAction");
      String[] cert = pem.getText().split("\n");
      String aliasValue = alias.getText();
      getUiHandlers().addNewCertificate(aliasValue, cert, getCertificateOption());
   }
   
   /**
    * A click button handler attached to the "Cancel" button.
    * 
    * @param    e
    *           Click event
    */
   @UiHandler("cancelAction")
   void onCancelAction(ClickEvent e)
   {
      logger.log(Level.INFO, "onCancelAction");
   }

   /**
    * Provides an access to the target modal dialog.
    * 
    * @return   The underlined modal dialog
    */
   @Override
   protected Modal getModal()
   {
      return modal;
   }
   
   /**
    * It hides this dialog.
    */
   public void hide()
   {
      getModal().hide();
   }
   
   /**
    * Read the target file.
    * 
    * @param    e
    *           The native event generated if the target file had been selected to upload.
    * @param    textArea
    *           The  text area to display the target PEM certificate
    */
   private native void readFile(NativeEvent e, Element textArea)
   /*-{
      if (e.target && e.target.files && e.target.files.length && e.target.files.length > 0)
      {
         var reader = new FileReader();
         reader.onload = function(evt)
         {
            if (evt.target.result)
            {
               $wnd.jQuery(textArea).val(evt.target.result);
            }
         };
         reader.readAsText(e.target.files[0]);
      }
   }-*/;
   
   /**
    * Clears the target input element.
    * 
    * @param    input
    *           The target input element.
    */
   private native void resetField(Element input)
   /*-{
      if (input.value)
      {
         input.value = "";
      }
   }-*/;
}
