/*
** Module   : CertificatesUIHandlers.java
** Abstract : CertificatesUIHandlers provides all business methods to work with certificates.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client.application.home.accounts.certificates;

import java.util.Set;

import com.goldencode.p2j.admin.TaggedName;
import com.goldencode.p2j.admin.client.application.home.accounts.CertificatesOptions;
import com.gwtplatform.mvp.client.UiHandlers;

/**
 * CertificatesUIHandlers is a business interface for UI views to use business methods to work
 * with certificates.
 */
public interface CertificatesUIHandlers extends UiHandlers
{
   /**
    * This method is called if certificate's options are changed.
    * 
    * @param    value
    *           New certificate role.
    * @param    unusedOnly
    *           The new value of the unused checkbox.
    */
   void certificatesOptionsChanged(CertificatesOptions value, boolean unusedOnly);
   
   /**
    * Refreshes list of certificates having these options.
    * 
    * @param    value
    *           New certificate role to display only certificates with this role.
    * @param    unusedOnly
    *           The unused checkbox value to display only unused(Unassigned) certificates.
    */
   void onRefreshCertificatesList(CertificatesOptions value, boolean unusedOnly);
   
   /**
    * Adds new PEM certificate.
    * 
    * @param    alias
    *           The certificate name
    * @param    cert
    *           The array of certificate PEM strings
    * @param    value
    *           The certificate role.
    */
   void addNewCertificate(String alias, String[] cert, CertificatesOptions value);
   
   /**
    * These certificates having the provided options have been selected by a user.
    * 
    * @param    selected
    *           The array of certificates selected by a user
    * @param    option
    *           The current certificate role.
    * @param    unusedOnly
    *           The current unused checkbox value.
    */
   void onCertsSelected(TaggedName[] selected, CertificatesOptions option, boolean unusedOnly);

   /**
    * Notifies this set of selected certificates to delete.
    * 
    * @param    selected
    *           The selected certificates.
    */
   void onDeleteSelectedCertificates(Set<TaggedName> selected);

   /**
    * Change alias for the selected certificate.
    * 
    * @param    selected
    *           The selected certificate
    * @param    newAlias
    *           New certificate alias (name)
    */
   void changeAlias(TaggedName selected, String newAlias);

   /**
    * View details of the given selected certificate.
    *  
    * @param    selected
    *           The selected certificate
    */
   void viewCertificate(TaggedName selected);
}
