/*
** Module   : ProcessesUIHandlers.java
** Abstract : ProcessesUIHandlers defines common business methods to be used for process accounts
**            management.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/


package com.goldencode.p2j.admin.client.application.home.accounts.processes;

import java.io.Serializable;
import java.util.Set;

import com.goldencode.p2j.admin.ProcessDef;
import com.goldencode.p2j.admin.TaggedName;
import com.goldencode.p2j.admin.client.application.home.accounts.AccountsUiHandlers;


/**
 * ProcessesUIHandlers defines common business methods to be used by views for process accounts
 * management.
 */
public interface ProcessesUIHandlers
extends AccountsUiHandlers
{
   /**
    * Handles the action to refresh the table of process accounts.
    */
   void onRefreshProcesses();
   
   /**
    * Handles the action to refresh the table of certificates.
    */
   void onRefreshCerts();
   
   /**
    * Handles the process accounts selection changed action.
    * 
    * @param    selected
    *           The set of selected process accounts given by tagged names
    */
   void onProcessSelectionChanged(Set<TaggedName> selected);
   
   /**
    * Handles the certificate selection changed action.
    * 
    * @param    selectedCert
    *           The selected certificate
    */
   void onCertificateSelectionChanged(TaggedName selectedCert);
   
   /**
    * Executes the action to disable the given process account.
    * 
    * @param    ps
    *           The process account given by its tagged name
    */
   void disableProcessAccount(TaggedName ps);
   
   /**
    * Executes the action to delete the given process accounts
    * 
    * @param    accounts
    *           The process accounts to be deleted
    */
   void deleteProcessAccounts(Set<TaggedName> accounts);
   
   /**
    * Executes the action to assign the selected certificate to the selected process account.
    * 
    * @param    selectedCert
    *           The selected certificate given by its tagged name
    * @param    selectedProcess
    *           The selected process account given by its tagged name
    */
   void assignCertificateToProcess(TaggedName selectedCert, TaggedName selectedProcess);
   
   /**
    * Adds the new process account.
    * 
    * @param    accountDef
    *           The process account definition
    * @param    ext
    *           The account extension object
    */
   void addProcess(ProcessDef accountDef, Serializable ext);
   
   /**
    * Updates the given process account.
    * 
    * @param    accountDef
    *           The given process account definition
    * @param    ext
    *           The given account extension object
    */
   void updateProcess(ProcessDef accountDef, Serializable ext);
   
   /**
    * Clones the process account with its ACL if it is chosen.
    * 
    * @param    nameFrom
    *           The cloning process account name
    * @param    nameTo
    *           The new cloned process account name
    * @param    acls
    *           The flag that requests to clone the process account ACL
    */
   void cloneProcess(String nameFrom, String nameTo, boolean acls);
   
   /**
    * Shows the Add Process Account Dialog.
    */
   void showAddProcessAccountDialog();

   /**
    * Cancels the Add Process Account Dialog.
    */
   void cancelAddProcessAccountDialog();
   
   /**
    * Shows the Edit Process Account Dialog.
    * 
    * @param    selected
    *           The process account given by its tagged name
    */
   void showEditProcessAccountDialog(TaggedName selected);
   
   /**
    * Cancels the Edit Process Account Dialog.
    */
   void cancelEditProcessAccountDialog();
   
   /**
    * Setups the Clone Process Account Dialog.
    * 
    * @param    selected
    *           The process account given by its tagged name
    */
   void setupCloneProcessAccountDialog(TaggedName selected);
   
   /**
    * Cancels the Clone Process Account Dialog.
    */
   void cancelCloneProcessAccountDialog();
   
   /**
    * Executes the action to switch to the Certificate Management view.
    * 
    * @param    selected
    *           The certificate given by its tagged name
    */
   void switchToCertificates(TaggedName selected);
}
