/*
** Module   : FindUserAccount.java
** Abstract : FindUserAccount assembles the Find User Account dialog.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170914 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/


package com.goldencode.p2j.admin.client.application.home.accounts.users;

import java.util.Collections;
import java.util.Map;
import java.util.function.Consumer;

import org.gwtbootstrap3.client.ui.TextBox;

import com.goldencode.p2j.admin.client.*;
import com.goldencode.p2j.admin.client.widget.dialog.*;
import com.goldencode.p2j.admin.client.widget.dialog.InputDialog.*;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.event.dom.client.KeyPressEvent;
import com.google.gwt.event.dom.client.KeyPressHandler;
import com.google.gwt.user.client.ui.*;
import com.google.inject.*;

/**
 * Assembles the Find User Account dialog.
 */
public class FindUserAccount
{
   /** The account extension */
   @Inject(optional = true)
   private final AccountExtension acctExtension;

   /**
    * Creates the Find User Account manager.
    * 
    * @param    acctExtension
    *           The account extension holder
    */
   @Inject
   public FindUserAccount(AccountExtensionHolder acctExtension)
   {
      this.acctExtension = acctExtension.value;
   }

   /**
    * Shows the Find User Account dialog.
    * 
    * @param    parent
    *           The parent container for the Find User Account dialog
    * @param    dialog
    *           The template dialog to create the Find User Account dialog
    * @param    findAccount
    *           The implementation of the find user account use case
    */
   public void show(HasWidgets parent, InputDialog dialog, IFindAccountScreen findAccount)
   {
      Consumer<Map<Field, Object>> actionHandler;
      // Builds fields of the Find User Account dialog.
      if (acctExtension != null)
      {
         actionHandler = acctExtension.buildUserFindDialog(dialog, findAccount);
      }
      else
      {
         actionHandler = buildUserFindDialog(dialog, findAccount);
      }
      
      dialog.show(parent, false, actionHandler);
   }
   
   /**
    * Builds the Find User Account dialog and returns the dialog action handler.
    * 
    * @param    dialog
    *           The template dialog to create the Find User Account dialog
    * @param    findAccount
    *           The implementation of the find user account use case
    * 
    * @return   The dialog's action handler
    */
   private Consumer<Map<Field, Object>> buildUserFindDialog(InputDialog dialog,
                                                            IFindAccountScreen findAccount)
   {
      Field acctName = new Field("Enter User Name", "").withFieldSize(4);
      dialog.setSaveButtonText("Find");
      dialog.build("Find User", acctName);
      TextBox inputField = (TextBox) dialog.getWidget(acctName);
      inputField.setFocus(true);
      
      Consumer<Map<Field, Object>> actionHandler = fieldValueMap -> {
         if (fieldValueMap != null)
         {
            String value = inputField.getValue();
            findAccount.findAccountByName(value);
         }
         else
         {
            // cancel dialog
            dialog.dismiss();
         }
      };
      
      inputField.addKeyPressHandler(new KeyPressHandler()
      {
         @Override
         public void onKeyPress(KeyPressEvent event)
         {
            if (event.getNativeEvent().getKeyCode() == KeyCodes.KEY_ENTER)
            {
               Object value = dialog.getWidget(acctName).getValue();
               Map<Field, Object> m = Collections.<Field, Object>singletonMap(acctName, value);
               actionHandler.accept(m);
            }
         }
      });
      
      return actionHandler;
   }
}
