/*
** Module   : ListSelect.java
** Abstract : Selectable list widget.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 HC  20170612 Initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client.application.home.runtime.customlib;

import com.goldencode.p2j.admin.*;
import com.goldencode.p2j.admin.client.widget.*;
import com.goldencode.p2j.admin.client.widget.dialog.*;
import com.goldencode.p2j.admin.shared.*;
import com.google.gwt.event.dom.client.*;
import com.google.gwt.uibinder.client.*;
import com.google.gwt.user.cellview.client.*;
import com.google.gwt.user.client.ui.*;
import com.google.gwt.view.client.*;
import com.google.inject.*;
import org.gwtbootstrap3.client.ui.*;
import org.gwtbootstrap3.client.ui.Button;
import org.gwtbootstrap3.client.ui.gwt.*;
import org.gwtbootstrap3.client.ui.gwt.DataGrid;
import org.gwtbootstrap3.client.ui.html.*;

import java.util.*;
import java.util.function.*;

/**
 * Selectable list widget.
 */
public class ListSelect
extends Composite
{
   /**
    * GWT binder.
    */
   interface Binder
   extends UiBinder<Modal, ListSelect>
   {}

   /** The list grid */
   @UiField(provided = true)
   DataGrid<TaggedName> grid = new DataGrid<>(Integer.MAX_VALUE);

   /** Modal dialog this view will be displayed in */
   @UiField
   Modal modal;

   /** The modal dialog footer */
   @UiField
   ModalFooter modalFooter;

   /** Widget */
   @UiField
   Button bOK;

   /** Widget */
   @UiField
   Button bCancel;

   /** Widget */
   @UiField
   Text helpText;

   /** Widget */
   @UiField
   Heading gridTitle;

   /** Dialog close handler */
   private Consumer<TaggedName> closeHandler;

   /** List grid handle */
   private GridHandle<TaggedName> gridHandle;

   /**
    * Ctor.
    *
    * @param   binder
    *          GWT binder.
    */
   @Inject
   ListSelect(ListSelect.Binder binder)
   {
      initWidget(binder.createAndBindUi(this));
   }

   /**
    * Configure the dialog.
    *
    * @param parent
    *        Parent widget where this widget will be revealed in.
    * @param data
    *        List data.
    * @param dialogTitle
    *        Dialog title.
    * @param tableTitle
    *        Table title.
    * @param columns
    *        Array of column names.
    * @param helpText
    *        Help text to be displayed above the table or <code>null</code>
    *        if it is not needed.
    * @param closeHandler
    *        Invoked when the dialog is closed.
    */
   public void show(HasWidgets parent,
                    TaggedName[] data,
                    String dialogTitle,
                    String tableTitle,
                    String[] columns,
                    String helpText,
                    Consumer<TaggedName> closeHandler)
   {
      show(parent, data, dialogTitle, tableTitle, columns, helpText, null, closeHandler);
   }

   /**
    * Configure the dialog.
    *
    * @param parent
    *        Parent widget where this widget will be revealed in.
    * @param data
    *        List data.
    * @param dialogTitle
    *        Dialog title.
    * @param tableTitle
    *        Table title.
    * @param columns
    *        Array of column names.
    * @param helpText
    *        Help text to be displayed above the table or <code>null</code>
    *        if it is not needed.
    * @param selectRow
    *        The object representing the row to select.
    * @param closeHandler
    *        Invoked when the dialog is closed.
    */
   public void show(HasWidgets parent,
                    TaggedName[] data,
                    String dialogTitle,
                    String tableTitle,
                    String[] columns,
                    String helpText,
                    TaggedName selectRow,
                    Consumer<TaggedName> closeHandler)
   {
      this.modal.addHiddenHandler(evt ->
      {
         if (closeHandler != null)
         {
            closeHandler.accept(null);
         }
      });

      this.closeHandler = closeHandler;
      modal.setTitle(dialogTitle);
      this.helpText.setText(helpText);
      gridTitle.setText(tableTitle);

      GridHelper.ColumnInfo[] gridCols = new GridHelper.ColumnInfo[columns.length];
      for (int i = 0; i < columns.length; i++)
      {
         int finalI = i;
         @SuppressWarnings("NonJREEmulationClassesInClientCode")
         GridHelper.ColumnInfo col =
            new GridHelper.ColumnInfo<TaggedName>(
               columns[i],
               tn -> tn.get(finalI),
               (tn1, tn2) -> tn1.get(finalI).compareTo(tn2.get(finalI)));
         gridCols[i] = col;
      }

      parent.add(this);
      modal.addShownHandler(evt ->
      {
         // create the grid in the modal shown handler, this is because the grid layout
         // logic needs to know the element's client height, which is non-zero only after
         // the modal is made visible
         gridHandle = GridHelper.initListGrid(grid, gridCols);
         gridHandle.getSelectionModel().addSelectionChangeHandler(event ->
         {
            bOK.setEnabled(gridHandle.getSelectedSingle() != null);
         });
         gridHandle.getDataProvider().getList().addAll(Arrays.asList(data));

         if (selectRow != null)
         {
            gridHandle.getSelectionModel().setSelected(selectRow, true);
         }
      });

      modal.show();
   }

   /**
    * Click event handler.
    *
    * @param   event
    *          Click event.
    */
   @UiHandler("bOK")
   public void bOKClick(ClickEvent event)
   {
      close(false);
   }

   /**
    * Click event handler.
    *
    * @param   event
    *          Click event.
    */
   @UiHandler("bCancel")
   public void bCancelClick(ClickEvent event)
   {
      close(true);
   }

   /**
    * Closes the dialog.
    *
    * @param   cancel
    *          When <code>true</code> the selected record will not be passed to the done handler.
    */
   private void close(boolean cancel)
   {
      TaggedName selected = cancel ? null : gridHandle.getSelectedSingle();
      Consumer<TaggedName> closeHandler = this.closeHandler;
      // clear the handler so that it is not fired again in the modal hidden handler
      this.closeHandler = null;
      modal.hide();
      removeFromParent();
      closeHandler.accept(selected);
   }
}
