/*
** Module   : MessageDialogView.java
** Abstract : Message modal dialog.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 HC  20170612 Initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.admin.client.widget.dialog;

import com.google.gwt.dom.client.*;
import com.google.gwt.uibinder.client.*;
import com.google.gwt.user.client.ui.*;
import com.google.inject.*;
import com.google.web.bindery.event.shared.*;
import com.gwtplatform.mvp.client.*;
import org.gwtbootstrap3.client.ui.Button;
import org.gwtbootstrap3.client.ui.*;
import org.gwtbootstrap3.client.ui.constants.*;
import org.gwtbootstrap3.client.ui.html.*;

import java.util.function.*;

/**
 * Message modal dialog.
 */
public class MessageDialogView
extends ViewImpl
implements MessageDialogPresenter.MyView
{
   /**
    * GWT binder.
    */
   interface Binder
   extends UiBinder<Modal, MessageDialogView>
   {
   }

   /** The modal window */
   @UiField
   Modal modal;

   /** The icon displayed in the dialog */
   @UiField
   Icon icon;

   /** The modal title */
   @UiField
   Span title;

   /** Dialog message */
   @UiField
   HTML bodyMessage;

   /** Modal footer */
   @UiField
   ModalFooter modalFooter;

   /** Modal dialog dismiss event registration */
   private HandlerRegistration hideHandlerReg;

   /**
    * Ctor.
    *
    * @param   binder
    *          GWT binder.
    */
   @Inject
   MessageDialogView(MessageDialogView.Binder binder)
   {
      initWidget(binder.createAndBindUi(this));
   }

   /**
    * Called when the view is about to be revealed.
    */
   @Override
   public void onReveal()
   {
   }

   /**
    * Shows the message dialog.
    *
    * @param   title
    *          Dialog title.
    * @param   message
    *          Message dialog message.
    * @param   icon
    *          Message dialog icon.
    * @param   buttonLabels
    *          Button labels.
    * @param   buttonTypes
    *          Button types.
    * @param   closeCallback
    *          Close callback.
    */
   @Override
   public void show(String title,
                    String message,
                    IconType icon,
                    String[] buttonLabels,
                    ButtonType[] buttonTypes,
                    Consumer<Integer> closeCallback)
   {
      if (buttonLabels.length != buttonTypes.length)
      {
         throw new RuntimeException("Button label array length must be equal to button types array!");
      }

      if (icon == null)
      {
         this.icon.setVisible(false);
      }
      else
      {
         this.icon.setVisible(true);
         this.icon.setType(icon);
         this.icon.setSize(IconSize.TIMES2);
      }
      this.title.setHTML(title);
      bodyMessage.setHTML(message);
      modalFooter.clear();

      boolean activeModal = Document.get().getBody().hasClassName("modal-open");

      for (int i = 0; i < buttonLabels.length; i++)
      {
         String l = buttonLabels[i];
         ButtonType t = buttonTypes[i];
         Button b = new Button();
         b.setText(l);
         b.setType(t);
         modalFooter.add(b);
         int buttonIndex = i;
         b.addClickHandler(event ->
         {
            if (closeCallback != null)
            {
               hideHandlerReg = modal.addHiddenHandler(evt ->
               {
                  hideHandlerReg.removeHandler();

                  if (activeModal)
                  {
                     // when stack of modals are displayed the modal-open style must be restored
                     // so that the modal under this one has the correct appearance (for example
                     // the scroll bars are displayed and the modal can be scrolled when larger
                     // than the visible area)
                     Document.get().getBody().addClassName("modal-open");
                  }

                  closeCallback.accept(buttonIndex);
               });
            }
            modal.hide();
         });
      }

      modal.show();
   }
}