/*
** Module   : AdminServiceAsync.java
** Abstract : AdminServiceAsync defines the client peer interface of the remote administration
**            interface.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ------------------------------Description----------------------------------
** 001 HC  20170116 Initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/


package com.goldencode.p2j.admin.shared;

import java.io.Serializable;
import java.util.Date;
import java.util.Map;

import com.goldencode.p2j.admin.*;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.view.client.Range;


/**
 * Defines the client peer interface of the remote administration interface.
 */
public interface AdminServiceAsync
{
   
   /**
    * Gets the server name.
    *
    * @param    callback
    *           The callback
    */
   void getServerName(AsyncCallback<String> callback);

   /**
    * Checks if is master.
    *
    * @param    callback
    *           The callback
    */
   void isMaster(AsyncCallback<Boolean> callback);

   /**
    * Sets the target live.
    *
    * @param    stayCurrent
    *           the stay current
    * @param    callback
    *           The callback
    */
   void setTargetLive(boolean stayCurrent, AsyncCallback<Integer> callback);

   /**
    * Sets the target file.
    *
    * @param    name
    *           the name
    * @param    create
    *           the create
    * @param    callback
    *           The callback
    */
   void setTargetFile(String name, boolean create, AsyncCallback<Boolean> callback);

   /**
    * Target refresh.
    *
    * @param    callback
    *           The callback
    */
   void targetRefresh(AsyncCallback<Integer> callback);

   /**
    * Checks if is refresh pending.
    *
    * @param    callback
    *           The callback
    */
   void isRefreshPending(AsyncCallback<Boolean> callback);

   /**
    * Can refresh.
    *
    * @param    callback
    *           The callback
    */
   void canRefresh(AsyncCallback<Boolean> callback);

   /**
    * Gets the messages.
    *
    * @param    all
    *           the all
    * @param    callback
    *           The callback
    */
   void getMessages(boolean all, AsyncCallback<String[][]> callback);

   /**
    * Peek message.
    *
    * @param    callback
    *           The callback
    */
   void peekMessage(AsyncCallback<String> callback);

   /**
    * Clear messages.
    *
    * @param    callback
    *           The callback
    */
   void clearMessages(AsyncCallback<Void> callback);

   /**
    * Gets the admin def.
    *
    * @param    callback
    *           The callback
    */
   void getAdminDef(AsyncCallback<AdminDef> callback);

   /**
    * Gets the profile.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void getProfile(String[] name, AsyncCallback<AdminProfile[]> callback);

   /**
    * Gets the profile.
    *
    * @param    name
    *           the name
    * @param    perm
    *           the perm
    * @param    callback
    *           The callback
    */
   void getProfile(String name, int[] perm, AsyncCallback<AdminProfile[]> callback);

   /**
    * Backup directory.
    *
    * @param    newfile
    *           the newfile
    * @param    callback
    *           The callback
    */
   void backupDirectory(String newfile, AsyncCallback<Boolean> callback);

   /**
    * Lock for reread dir.
    *
    * @param    seconds
    *           the seconds
    * @param    callback
    *           The callback
    */
   void lockForRereadDir(int seconds, AsyncCallback<Boolean> callback);

   /**
    * Reread directory.
    *
    * @param    callback
    *           The callback
    */
   void rereadDirectory(AsyncCallback<Integer> callback);

   /**
    * Cancel reread dir.
    *
    * @param    callback
    *           The callback
    */
   void cancelRereadDir(AsyncCallback<Void> callback);

   /**
    * Gets the session list.
    *
    * @param    callback
    *           The callback
    */
   void getSessionList(AsyncCallback<SessionInfo[]> callback);

   /**
    * Terminate session.
    *
    * @param    sid
    *           the sid
    * @param    callback
    *           The callback
    */
   void terminateSession(int sid, AsyncCallback<Void> callback);

   /**
    * Gets the record lock list.
    *
    * @param    callback
    *           The callback
    */
   void getRecordLockList(AsyncCallback<RecordLockInfo[]> callback);

   /**
    * Gets the databases.
    *
    * @param    callback
    *           The callback
    */
   void getDatabases(AsyncCallback<String[]> callback);

   /**
    * Gets the database tables.
    *
    * @param    database
    *           the database
    * @param    callback
    *           The callback
    */
   void getDatabaseTables(String database, AsyncCallback<String[]> callback);

   /**
    * Lock records.
    *
    * @param    records
    *           the records
    * @param    callback
    *           The callback
    */
   void lockRecords(RecordInfo[] records, AsyncCallback<Boolean> callback);

   /**
    * Unlock records.
    *
    * @param    callback
    *           The callback
    */
   void unlockRecords(AsyncCallback<Boolean> callback);

   /**
    * Gets the lock records.
    *
    * @param    callback
    *           The callback
    */
   void getLockRecords(AsyncCallback<RecordInfo[]> callback);
   
   /**
    * Shutdown.
    *
    * @param    callback
    *           The callback
    */
   void shutdown(AsyncCallback<Boolean> callback);

   /**
    * List resource names.
    *
    * @param    callback
    *           The callback
    */
   void listResourceNames(AsyncCallback<String[]> callback);

   /**
    * Gets the resource type.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void getResourceType(String name, AsyncCallback<String> callback);

   /**
    * Change resource names.
    *
    * @param    additions
    *           the additions
    * @param    deletions
    *           the deletions
    * @param    callback
    *           The callback
    */
   void changeResourceNames(String[] additions, String[] deletions,
            AsyncCallback<Boolean> callback);

   /**
    * Gets the auth config.
    *
    * @param    callback
    *           The callback
    */
   void getAuthConfig(AsyncCallback<AuthConfigDef> callback);

   /**
    * Sets the auth config.
    *
    * @param    authConfig
    *           the auth config
    * @param    callback
    *           The callback
    */
   void setAuthConfig(AuthConfigDef authConfig, AsyncCallback<Boolean> callback);

   /**
    * List holidays.
    *
    * @param    callback
    *           The callback
    */
   void listHolidays(AsyncCallback<Date[]> callback);

   /**
    * Change holidays.
    *
    * @param    additions
    *           the additions
    * @param    deletions
    *           the deletions
    * @param    callback
    *           The callback
    */
   void changeHolidays(Date[] additions, Date[] deletions, AsyncCallback<Boolean> callback);

   /**
    * Gets the audit.
    *
    * @param    callback
    *           The callback
    */
   void getAudit(AsyncCallback<AuditDef> callback);

   /**
    * Sets the audit.
    *
    * @param    audit
    *           the audit
    * @param    callback
    *           The callback
    */
   void setAudit(AuditDef audit, AsyncCallback<Boolean> callback);

   /**
    * List groups.
    *
    * @param    callback
    *           The callback
    */
   void listGroups(AsyncCallback<TaggedName[]> callback);

   /**
    * Check group.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void checkGroup(String name, AsyncCallback<TaggedName> callback);

   /**
    * Gets the group.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void getGroup(String name, AsyncCallback<GroupDef> callback);

   /**
    * Sets the group.
    *
    * @param    group
    *           the group
    * @param    callback
    *           The callback
    */
   void setGroup(GroupDef group, AsyncCallback<Boolean> callback);

   /**
    * Adds the group.
    *
    * @param    group
    *           the group
    * @param    callback
    *           The callback
    */
   void addGroup(GroupDef group, AsyncCallback<Boolean> callback);

   /**
    * Delete group.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void deleteGroup(String name, AsyncCallback<Boolean> callback);

   /**
    * Clone group.
    *
    * @param    nameFrom
    *           the name from
    * @param    nameTo
    *           the name to
    * @param    acls
    *           the acls
    * @param    callback
    *           The callback
    */
   void cloneGroup(String nameFrom, String nameTo, boolean acls, AsyncCallback<Boolean> callback);

   /**
    * List group users.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void listGroupUsers(String name, AsyncCallback<TaggedName[]> callback);

   /**
    * Change group users.
    *
    * @param    name
    *           the name
    * @param    additions
    *           the additions
    * @param    deletions
    *           the deletions
    * @param    callback
    *           The callback
    */
   void changeGroupUsers(String name, String[] additions, String[] deletions,
            AsyncCallback<Boolean> callback);

   /**
    * List users.
    *
    * @param    callback
    *           The callback
    */
   void listUsers(AsyncCallback<TaggedName[]> callback);

   /**
    * Check user.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void checkUser(String name, AsyncCallback<TaggedName> callback);

   /**
    * Gets the user.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void getUser(String name, AsyncCallback<UserDef> callback);

   /**
    * Sets the user.
    *
    * @param    user
    *           the user
    * @param    callback
    *           The callback
    */
   void setUser(UserDef user, AsyncCallback<Boolean> callback);

   /**
    * Sets the user.
    *
    * @param    user
    *           the user
    * @param    ext
    *           the ext
    * @param    callback
    *           The callback
    */
   void setUser(UserDef user, Serializable ext, AsyncCallback<Boolean> callback);

   /**
    * Adds the user.
    *
    * @param    user
    *           the user
    * @param    ext
    *           the ext
    * @param    callback
    *           The callback
    */
   void addUser(UserDef user, Serializable ext, AsyncCallback<Boolean> callback);

   /**
    * Delete user.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void deleteUser(String name, AsyncCallback<Boolean> callback);

   /**
    * Clone user.
    *
    * @param    nameFrom
    *           the name from
    * @param    nameTo
    *           the name to
    * @param    acls
    *           the acls
    * @param    callback
    *           The callback
    */
   void cloneUser(String nameFrom, String nameTo, boolean acls, AsyncCallback<Boolean> callback);

   /**
    * Clone user.
    *
    * @param    nameFrom
    *           the name from
    * @param    nameTo
    *           the name to
    * @param    acls
    *           the acls
    * @param    newPassword
    *           the new password
    * @param    callback
    *           The callback
    */
   void cloneUser(String nameFrom, String nameTo, boolean acls, String newPassword,
            AsyncCallback<Boolean> callback);

   /**
    * List user groups.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void listUserGroups(String name, AsyncCallback<TaggedName[]> callback);

   /**
    * List processes.
    *
    * @param    callback
    *           The callback
    */
   void listProcesses(AsyncCallback<TaggedName[]> callback);

   /**
    * Check process.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void checkProcess(String name, AsyncCallback<TaggedName> callback);

   /**
    * Gets the process.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void getProcess(String name, AsyncCallback<ProcessDef> callback);

   /**
    * Sets the process.
    *
    * @param    process
    *           the process
    * @param    callback
    *           The callback
    */
   void setProcess(ProcessDef process, AsyncCallback<Boolean> callback);

   /**
    * Sets the process.
    *
    * @param    process
    *           the process
    * @param    ext
    *           the ext
    * @param    callback
    *           The callback
    */
   void setProcess(ProcessDef process, Serializable ext, AsyncCallback<Boolean> callback);

   /**
    * Adds the process.
    *
    * @param    process
    *           the process
    * @param    ext
    *           the ext
    * @param    callback
    *           The callback
    */
   void addProcess(ProcessDef process, Serializable ext, AsyncCallback<Boolean> callback);

   /**
    * Delete process.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void deleteProcess(String name, AsyncCallback<Boolean> callback);

   /**
    * Clone process.
    *
    * @param    nameFrom
    *           the name from
    * @param    nameTo
    *           the name to
    * @param    acls
    *           the acls
    * @param    callback
    *           The callback
    */
   void cloneProcess(String nameFrom, String nameTo, boolean acls, AsyncCallback<Boolean> callback);

   /**
    * List auth certs.
    *
    * @param    unusedOnly
    *           the unused only
    * @param    callback
    *           The callback
    */
   void listAuthCerts(boolean unusedOnly, AsyncCallback<TaggedName[]> callback);

   /**
    * List peer certs.
    *
    * @param    unusedOnly
    *           the unused only
    * @param    callback
    *           The callback
    */
   void listPeerCerts(boolean unusedOnly, AsyncCallback<TaggedName[]> callback);

   /**
    * Gets the cert users.
    *
    * @param    alias
    *           the alias
    * @param    callback
    *           The callback
    */
   void getCertUsers(String alias, AsyncCallback<TaggedName[]> callback);

   /**
    * Gets the cert processes.
    *
    * @param    alias
    *           the alias
    * @param    callback
    *           The callback
    */
   void getCertProcesses(String alias, AsyncCallback<TaggedName[]> callback);

   /**
    * Gets the cert accounts.
    *
    * @param    alias
    *           the alias
    * @param    callback
    *           The callback
    */
   void getCertAccounts(String alias, AsyncCallback<TaggedName[]> callback);

   /**
    * Gets the cert.
    *
    * @param    alias
    *           the alias
    * @param    callback
    *           The callback
    */
   void getCert(String alias, AsyncCallback<CertDef> callback);

   /**
    * Sets the cert.
    *
    * @param    cert
    *           the cert
    * @param    callback
    *           The callback
    */
   void setCert(CertDef cert, AsyncCallback<Boolean> callback);

   /**
    * Adds the cert.
    *
    * @param    cert
    *           the cert
    * @param    callback
    *           The callback
    */
   void addCert(CertDef cert, AsyncCallback<Boolean> callback);

   /**
    * Move cert.
    *
    * @param    aliasFrom
    *           the alias from
    * @param    aliasTo
    *           the alias to
    * @param    callback
    *           The callback
    */
   void moveCert(String aliasFrom, String aliasTo, AsyncCallback<Boolean> callback);

   /**
    * Search cert.
    *
    * @param    dn
    *           the dn
    * @param    callback
    *           The callback
    */
   void searchCert(String dn, AsyncCallback<String> callback);

   /**
    * Delete cert.
    *
    * @param    alias
    *           the alias
    * @param    force
    *           the force
    * @param    callback
    *           The callback
    */
   void deleteCert(String alias, boolean force, AsyncCallback<Boolean> callback);

   /**
    * List acl instances.
    *
    * @param    callback
    *           The callback
    */
   void listAclInstances(AsyncCallback<String[]> callback);

   /**
    * List acl instances.
    *
    * @param    resourceType
    *           the resource type
    * @param    callback
    *           The callback
    */
   void listAclInstances(String resourceType, AsyncCallback<String[]> callback);

   /**
    * List instance acls.
    *
    * @param    instanceName
    *           the instance name
    * @param    callback
    *           The callback
    */
   void listInstanceAcls(String instanceName, AsyncCallback<String[]> callback);

   /**
    * Gets the acl.
    *
    * @param    resourceType
    *           the resource type
    * @param    instanceName
    *           the instance name
    * @param    callback
    *           The callback
    */
   void getAcl(String resourceType, String instanceName, AsyncCallback<AclDef> callback);

   /**
    * Gets the all acls.
    *
    * @param    instanceName
    *           the instance name
    * @param    callback
    *           The callback
    */
   void getAllAcls(String instanceName, AsyncCallback<AclDef[]> callback);

   /**
    * Gets the effective acls.
    *
    * @param    resourceType
    *           the resource type
    * @param    callback
    *           The callback
    */
   void getEffectiveAcls(String resourceType, AsyncCallback<AclDef> callback);

   /**
    * Adds the acl.
    *
    * @param    resourceType
    *           the resource type
    * @param    acl
    *           the acl
    * @param    subjMap
    *           the subj map
    * @param    callback
    *           The callback
    */
   void addAcl(String resourceType, Acl acl, Map<Integer, String> subjMap,
            AsyncCallback<Boolean> callback);

   /**
    * Sets the acl.
    *
    * @param    resourceType
    *           the resource type
    * @param    acl
    *           the acl
    * @param    subjMap
    *           the subj map
    * @param    setInstance
    *           the set instance
    * @param    setInstanceMode
    *           the set instance mode
    * @param    setSubjects
    *           the set subjects
    * @param    setRights
    *           the set rights
    * @param    callback
    *           The callback
    */
   void setAcl(String resourceType, Acl acl, Map<Integer, String> subjMap, boolean setInstance,
            boolean setInstanceMode, boolean setSubjects, boolean setRights,
            AsyncCallback<Boolean> callback);

   /**
    * Move acl.
    *
    * @param    resourceType
    *           the resource type
    * @param    instanceName
    *           the instance name
    * @param    oid
    *           the oid
    * @param    newInstanceName
    *           the new instance name
    * @param    callback
    *           The callback
    */
   void moveAcl(String resourceType, String instanceName, int oid, String newInstanceName,
            AsyncCallback<Boolean> callback);

   /**
    * Move acl.
    *
    * @param    resourceType
    *           the resource type
    * @param    instanceName
    *           the instance name
    * @param    oid
    *           the oid
    * @param    newOid
    *           the new oid
    * @param    callback
    *           The callback
    */
   void moveAcl(String resourceType, String instanceName, int oid, int newOid,
            AsyncCallback<Boolean> callback);

   /**
    * Delete acl.
    *
    * @param    resourceType
    *           the resource type
    * @param    instanceName
    *           the instance name
    * @param    oid
    *           the oid
    * @param    callback
    *           The callback
    */
   void deleteAcl(String resourceType, String instanceName, int oid,
            AsyncCallback<Boolean> callback);

   /**
    * Clone acl.
    *
    * @param    resourceType
    *           the resource type
    * @param    instanceName
    *           the instance name
    * @param    oid
    *           the oid
    * @param cloneOid the clone oid
    * @param    callback
    *           The callback
    */
   void cloneAcl(String resourceType, String instanceName, int oid, int cloneOid,
            AsyncCallback<Boolean> callback);

   /**
    * Resequence acls.
    *
    * @param    resourceType
    *           the resource type
    * @param    instanceName
    *           the instance name
    * @param    start
    *           the start
    * @param    step
    *           the step
    * @param    callback
    *           The callback
    */
   void resequenceAcls(String resourceType, String instanceName, int start, int step,
            AsyncCallback<Boolean> callback);

   /**
    * Adds the acl instance.
    *
    * @param    instanceName
    *           the instance name
    * @param    callback
    *           The callback
    */
   void addAclInstance(String instanceName, AsyncCallback<Boolean> callback);

   /**
    * Delete acl instance.
    *
    * @param    instanceName
    *           the instance name
    * @param    callback
    *           The callback
    */
   void deleteAclInstance(String instanceName, AsyncCallback<Boolean> callback);

   /**
    * Delete acl instance.
    *
    * @param    resourceType
    *           the resource type
    * @param    instanceName
    *           the instance name
    * @param    callback
    *           The callback
    */
   void deleteAclInstance(String resourceType, String instanceName,
            AsyncCallback<Boolean> callback);

   /**
    * Clone acl instance.
    *
    * @param    instanceFrom
    *           the instance from
    * @param    instanceTo
    *           the instance to
    * @param    callback
    *           The callback
    */
   void cloneAclInstance(String instanceFrom, String instanceTo, AsyncCallback<Boolean> callback);

   /**
    * Clone acl instance.
    *
    * @param    resourceType
    *           the resource type
    * @param    instanceFrom
    *           the instance from
    * @param    instanceTo
    *           the instance to
    * @param    callback
    *           The callback
    */
   void cloneAclInstance(String resourceType, String instanceFrom, String instanceTo,
            AsyncCallback<Boolean> callback);

   /**
    * Move acl instance.
    *
    * @param    instanceFrom
    *           the instance from
    * @param    instanceTo
    *           the instance to
    * @param    callback
    *           The callback
    */
   void moveAclInstance(String instanceFrom, String instanceTo, AsyncCallback<Boolean> callback);

   /**
    * Move acl instance.
    *
    * @param    resourceType
    *           the resource type
    * @param    instanceFrom
    *           the instance from
    * @param    instanceTo
    *           the instance to
    * @param    callback
    *           The callback
    */
   void moveAclInstance(String resourceType, String instanceFrom, String instanceTo,
            AsyncCallback<Boolean> callback);

   /**
    * List servers.
    *
    * @param    callback
    *           The callback
    */
   void listServers(AsyncCallback<String[]> callback);

   /**
    * List servers.
    *
    * @param    containers
    *           the containers
    * @param    callback
    *           The callback
    */
   void listServers(String containers, AsyncCallback<String[]> callback);

   /**
    * List auth plugins.
    *
    * @param    callback
    *           The callback
    */
   void listAuthPlugins(AsyncCallback<TaggedName[]> callback);

   /**
    * Bulk auth mode update.
    *
    * @param    targetAccounts
    *           the target accounts
    * @param    excludedAccounts
    *           the excluded accounts
    * @param    authMode
    *           the auth mode
    * @param    authPlugin
    *           the auth plugin
    * @param    callback
    *           The callback
    */
   void bulkAuthModeUpdate(int targetAccounts, String[] excludedAccounts, int authMode,
            String authPlugin, AsyncCallback<Integer> callback);

   /**
    * Gets the registered jars.
    *
    * @param    callback
    *           The callback
    */
   void getRegisteredJars(AsyncCallback<TaggedName[]> callback);

   /**
    * Gets the unregistered jars.
    *
    * @param    callback
    *           The callback
    */
   void getUnregisteredJars(AsyncCallback<String[]> callback);

   /**
    * Gets the available jar hook classes.
    *
    * @param    jar
    *           the jar
    * @param    callback
    *           The callback
    */
   void getAvailableJarHookClasses(String jar, AsyncCallback<String[]> callback);

   /**
    * Gets the stored jar hook class.
    *
    * @param    jar
    *           the jar
    * @param    callback
    *           The callback
    */
   void getStoredJarHookClass(String jar, AsyncCallback<String> callback);

   /**
    * Register jar.
    *
    * @param    jar
    *           the jar
    * @param hookClass the hook class
    * @param storeInDirectory the store in directory
    * @param    callback
    *           The callback
    */
   void registerJar(String jar, String hookClass, boolean storeInDirectory,
            AsyncCallback<JarOpErrorCode> callback);

   /**
    * Deregister jar.
    *
    * @param    jar
    *           the jar
    * @param    force
    *           the force
    * @param    removeFromDirectory
    *           the remove from directory
    * @param    callback
    *           The callback
    */
   void deregisterJar(String jar, boolean force, boolean removeFromDirectory,
            AsyncCallback<JarOpErrorCode> callback);

   /**
    * Deregister jar from directory.
    *
    * @param    jar
    *           the jar
    * @param    storedHookClass
    *           the stored hook class
    * @param    callback
    *           The callback
    */
   void deregisterJarFromDirectory(String jar, String storedHookClass,
            AsyncCallback<JarOpErrorCode> callback);

   /**
    * Change jar hook class.
    *
    * @param    jar
    *           the jar
    * @param    hookClass
    *           the hook class
    * @param    callback
    *           The callback
    */
   void changeJarHookClass(String jar, String hookClass, AsyncCallback<JarOpErrorCode> callback);

   /**
    * Gets the registered jar AP is.
    *
    * @param    jar
    *           the jar
    * @param    callback
    *           The callback
    */
   void getRegisteredJarAPIs(String jar, AsyncCallback<TaggedName[]> callback);

   /**
    * Gets the available jar API interfaces.
    *
    * @param    jar
    *           the jar
    * @param    showRegistered
    *           the show registered
    * @param    callback
    *           The callback
    */
   void getAvailableJarAPIInterfaces(String jar, boolean showRegistered,
            AsyncCallback<String[]> callback);

   /**
    * Gets the available jar API impl classes.
    *
    * @param    jar
    *           the jar
    * @param    interfaceClass
    *           the interface class
    * @param    callback
    *           The callback
    */
   void getAvailableJarAPIImplClasses(String jar, String interfaceClass,
            AsyncCallback<String[]> callback);

   /**
    * Register jar API.
    *
    * @param    interfaceClass
    *           the interface class
    * @param    implementationClass
    *           the implementation class
    * @param    callback
    *           The callback
    */
   void registerJarAPI(String interfaceClass, String implementationClass,
            AsyncCallback<JarOpErrorCode> callback);

   /**
    * Deregister jar API.
    *
    * @param    jar
    *           the jar
    * @param    interfaceClass
    *           the interface class
    * @param    force
    *           the force
    * @param    callback
    *           The callback
    */
   void deregisterJarAPI(String jar, String interfaceClass, boolean force,
            AsyncCallback<JarOpErrorCode> callback);

   /**
    * Change jar hooks type.
    *
    * @param    serverWide
    *           the server wide
    * @param    move
    *           the move
    * @param    callback
    *           The callback
    */
   void changeJarHooksType(boolean serverWide, boolean move,
            AsyncCallback<JarOpErrorCode> callback);

   /**
    * Gets the active sessions.
    *
    * @param    jar
    *           the jar
    * @param    apiInterfaceClass
    *           the api interface class
    * @param    callback
    *           The callback
    */
   void getActiveSessions(String jar, String apiInterfaceClass,
            AsyncCallback<SessionInfo[]> callback);

   /**
    * Checks if is using server wide hooks.
    *
    * @param    callback
    *           The callback
    */
   void isUsingServerWideHooks(AsyncCallback<Boolean> callback);

   /**
    * Checks if is using multi class loader.
    *
    * @param    callback
    *           The callback
    */
   void isUsingMultiClassLoader(AsyncCallback<Boolean> callback);

   /**
    * Gets the managed libs dir information.
    *
    * @param    callback
    *           The callback
    */
   void getManagedLibsDirInformation(AsyncCallback<TaggedName> callback);

   /**
    * List logging instances.
    *
    * @param    callback
    *           The callback
    */
   void listLoggingInstances(AsyncCallback<String[]> callback);

   /**
    * Gets the logging.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void getLogging(String name, AsyncCallback<LoggingDef> callback);

   /**
    * Sets the logging.
    *
    * @param    logging
    *           the logging
    * @param    callback
    *           The callback
    */
   void setLogging(LoggingDef logging, AsyncCallback<Boolean> callback);

   /**
    * Adds the logging.
    *
    * @param    name
    *           the name
    * @param    logging
    *           the logging
    * @param    callback
    *           The callback
    */
   void addLogging(String name, LoggingDef logging, AsyncCallback<Boolean> callback);

   /**
    * Clone logging.
    *
    * @param    nameFrom
    *           the name from
    * @param    nameTo
    *           the name to
    * @param    callback
    *           The callback
    */
   void cloneLogging(String nameFrom, String nameTo, AsyncCallback<Boolean> callback);

   /**
    * Move logging.
    *
    * @param    nameFrom
    *           the name from
    * @param    nameTo
    *           the name to
    * @param    callback
    *           The callback
    */
   void moveLogging(String nameFrom, String nameTo, AsyncCallback<Boolean> callback);

   /**
    * Delete logging.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void deleteLogging(String name, AsyncCallback<Boolean> callback);

   /**
    * List exports instances.
    *
    * @param    callback
    *           The callback
    */
   void listExportsInstances(AsyncCallback<String[]> callback);

   /**
    * Gets the exports.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void getExports(String name, AsyncCallback<ExportsDef> callback);

   /**
    * Sets the exports.
    *
    * @param    logging
    *           the logging
    * @param    callback
    *           The callback
    */
   void setExports(ExportsDef logging, AsyncCallback<Boolean> callback);

   /**
    * Adds the exports.
    *
    * @param    name
    *           the name
    * @param    logging
    *           the logging
    * @param    callback
    *           The callback
    */
   void addExports(String name, ExportsDef logging, AsyncCallback<Boolean> callback);

   /**
    * Clone exports.
    *
    * @param    nameFrom
    *           the name from
    * @param    nameTo
    *           the name to
    * @param    callback
    *           The callback
    */
   void cloneExports(String nameFrom, String nameTo, AsyncCallback<Boolean> callback);

   /**
    * Move exports.
    *
    * @param    nameFrom
    *           the name from
    * @param    nameTo
    *           the name to
    * @param    callback
    *           The callback
    */
   void moveExports(String nameFrom, String nameTo, AsyncCallback<Boolean> callback);

   /**
    * Delete exports.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void deleteExports(String name, AsyncCallback<Boolean> callback);

   /**
    * List startup instances.
    *
    * @param    callback
    *           The callback
    */
   void listStartupInstances(AsyncCallback<String[]> callback);

   /**
    * Gets the startup.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void getStartup(String name, AsyncCallback<StartupDef> callback);

   /**
    * Sets the startup.
    *
    * @param    startup
    *           the startup
    * @param    callback
    *           The callback
    */
   void setStartup(StartupDef startup, AsyncCallback<Boolean> callback);

   /**
    * Adds the startup.
    *
    * @param    name
    *           the name
    * @param    logging
    *           the logging
    * @param    callback
    *           The callback
    */
   void addStartup(String name, StartupDef logging, AsyncCallback<Boolean> callback);

   /**
    * Clone startup.
    *
    * @param    nameFrom
    *           the name from
    * @param    nameTo
    *           the name to
    * @param    callback
    *           The callback
    */
   void cloneStartup(String nameFrom, String nameTo, AsyncCallback<Boolean> callback);

   /**
    * Move startup.
    *
    * @param    nameFrom
    *           the name from
    * @param    nameTo
    *           the name to
    * @param    callback
    *           The callback
    */
   void moveStartup(String nameFrom, String nameTo, AsyncCallback<Boolean> callback);

   /**
    * Delete startup.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void deleteStartup(String name, AsyncCallback<Boolean> callback);

   /**
    * List persistence instances.
    *
    * @param    callback
    *           The callback
    */
   void listPersistenceInstances(AsyncCallback<String[]> callback);

   /**
    * Gets the persistence.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void getPersistence(String name, AsyncCallback<PersistenceDef> callback);

   /**
    * Sets the persistence.
    *
    * @param    persistence
    *           the persistence
    * @param    callback
    *           The callback
    */
   void setPersistence(PersistenceDef persistence, AsyncCallback<Boolean> callback);

   /**
    * Adds the persistence.
    *
    * @param    name
    *           the name
    * @param    persistence
    *           the persistence
    * @param    callback
    *           The callback
    */
   void addPersistence(String name, PersistenceDef persistence, AsyncCallback<Boolean> callback);

   /**
    * Clone persistence.
    *
    * @param    nameFrom
    *           the name from
    * @param    nameTo
    *           the name to
    * @param    callback
    *           The callback
    */
   void clonePersistence(String nameFrom, String nameTo, AsyncCallback<Boolean> callback);

   /**
    * Move persistence.
    *
    * @param    nameFrom
    *           the name from
    * @param    nameTo
    *           the name to
    * @param    callback
    *           The callback
    */
   void movePersistence(String nameFrom, String nameTo, AsyncCallback<Boolean> callback);

   /**
    * Delete persistence.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void deletePersistence(String name, AsyncCallback<Boolean> callback);

   /**
    * List database instances.
    *
    * @param    callback
    *           The callback
    */
   void listDatabaseInstances(AsyncCallback<String[]> callback);

   /**
    * Gets the database.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void getDatabase(String name, AsyncCallback<DatabaseDef> callback);

   /**
    * Sets the database.
    *
    * @param    database
    *           the database
    * @param    callback
    *           The callback
    */
   void setDatabase(DatabaseDef database, AsyncCallback<Boolean> callback);

   /**
    * Adds the database.
    *
    * @param    name
    *           the name
    * @param    logging
    *           the logging
    * @param    callback
    *           The callback
    */
   void addDatabase(String name, DatabaseDef logging, AsyncCallback<Boolean> callback);

   /**
    * Clone database.
    *
    * @param    nameFrom
    *           the name from
    * @param    nameTo
    *           the name to
    * @param    callback
    *           The callback
    */
   void cloneDatabase(String nameFrom, String nameTo, AsyncCallback<Boolean> callback);

   /**
    * Move database.
    *
    * @param    nameFrom
    *           the name from
    * @param    nameTo
    *           the name to
    * @param    callback
    *           The callback
    */
   void moveDatabase(String nameFrom, String nameTo, AsyncCallback<Boolean> callback);

   /**
    * Delete database.
    *
    * @param    name
    *           the name
    * @param    callback
    *           The callback
    */
   void deleteDatabase(String name, AsyncCallback<Boolean> callback);

   /**
    * List runtime names.
    *
    * @param    callback
    *           The callback
    */
   void listRuntimeNames(AsyncCallback<String[]> callback);

   /**
    * List runtime servers.
    *
    * @param    callback
    *           The callback
    */
   void listRuntimeServers(AsyncCallback<String[]> callback);

   /**
    * List runtime accounts.
    *
    * @param    callback
    *           The callback
    */
   void listRuntimeAccounts(AsyncCallback<String[]> callback);

   /**
    * Search runtime by account.
    *
    * @param    account
    *           the account
    * @param    itemName
    *           the item name
    * @param    callback
    *           The callback
    */
   void searchRuntimeByAccount(String account, String itemName, AsyncCallback<String[]> callback);

   /**
    * Gets the runtime.
    *
    * @param    serverName
    *           the server name
    * @param    accountName
    *           the account name
    * @param    callback
    *           The callback
    */
   void getRuntime(String serverName, String accountName, AsyncCallback<RuntimeDef> callback);

   /**
    * Sets the runtime.
    *
    * @param    runtime
    *           the runtime
    * @param    callback
    *           The callback
    */
   void setRuntime(RuntimeDef runtime, AsyncCallback<Boolean> callback);

   /**
    * Adds the runtime.
    *
    * @param    serverName
    *           the server name
    * @param    accountName
    *           the account name
    * @param    runtime
    *           the runtime
    * @param    callback
    *           The callback
    */
   void addRuntime(String serverName, String accountName, RuntimeDef runtime,
            AsyncCallback<Boolean> callback);

   /**
    * Clone runtime.
    *
    * @param    serverNameFrom
    *           the server name from
    * @param    accountNameFrom
    *           the account name from
    * @param    serverNameTo
    *           the server name to
    * @param    accountNameTo
    *           the account name to
    * @param    callback
    *           The callback
    */
   void cloneRuntime(String serverNameFrom, String accountNameFrom, String serverNameTo,
            String accountNameTo, AsyncCallback<Boolean> callback);

   /**
    * Move runtime.
    *
    * @param    serverNameFrom
    *           the server name from
    * @param    accountNameFrom
    *           the account name from
    * @param    serverNameTo
    *           the server name to
    * @param    accountNameTo
    *           the account name to
    * @param    callback
    *           The callback
    */
   void moveRuntime(String serverNameFrom, String accountNameFrom, String serverNameTo,
            String accountNameTo, AsyncCallback<Boolean> callback);

   /**
    * Delete runtime.
    *
    * @param    serverName
    *           the server name
    * @param    accountName
    *           the account name
    * @param    callback
    *           The callback
    */
   void deleteRuntime(String serverName, String accountName, AsyncCallback<Boolean> callback);

   /**
    * Gets the certificate hierarchy.
    *
    * @param    alias
    *           the alias
    * @param    callback
    *           The callback
    */
   public void getCertificateHierarchy(String alias, AsyncCallback<TaggedName[]> callback);

   /**
    * Validate certificate.
    *
    * @param    cert
    *           the cert
    * @param    callback
    *           The callback
    */
   public void validateCertificate(CertDef cert, AsyncCallback<Integer> callback);

   /**
    * Gets the certificate DN properties.
    *
    * @param    cd
    *           the cd
    * @param    owner
    *           the owner
    * @param    callback
    *           The callback
    */
   public void getCertificateDNProperties(CertDef cd, boolean owner, AsyncCallback<Map<String, String>> callback);

   /**
    * Gets the certificate DN property.
    *
    * @param    cd
    *           the cd
    * @param    prop
    *           the prop
    * @param    owner
    *           the owner
    * @param    callback
    *           The callback
    */
   public void getCertificateDNProperty(CertDef cd, String prop, boolean owner, AsyncCallback<String> callback);

   /**
    * Gets the certificate properties.
    *
    * @param    cd
    *           the cd
    * @param    callback
    *           The callback
    */
   public void getCertificateProperties(CertDef cd, AsyncCallback<Map<String, Object>> callback);

   /**
    * Checks if is self signed.
    *
    * @param    cert
    *           the cert
    * @param    callback
    *           The callback
    */
   public void isSelfSigned(CertDef cert, AsyncCallback<Boolean> callback);

   /**
    * Gets the preview images.
    *
    * @param    reportRequest
    *           the report request
    * @param    range
    *           the range
    * @param    callback
    *           The callback
    */
   public void getPreviewImages(ReportRequest reportRequest,
            Range range, AsyncCallback<ReportPreview> callback);

   /**
    * Returns the map of report parameters to their values saved in the directory.
    * 
    * @param    callback
    *           The callback
    */
   public void getReportParametersMap(AsyncCallback<Map<String, Object>> callback);

   /**
    * Gets the version info.
    *
    * @param    callback
    *           The callback
    */
   public void getVersionInfo(AsyncCallback<String> callback);

   public void forceFloatSerialization(AsyncCallback<Float> callback);
}
