/*
** Module   : PaperFormat.java
** Abstract : PaperFormat defines standard paper sizes.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description----------------------
** 001 SBI 20170601 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/


package com.goldencode.p2j.admin.shared;


/**
 * PaperFormat defines standard paper sizes.
 */
public enum PaperFormat
{
   /** LETTER */
   LETTER("Letter, 8.5\" x 11\"", 8.5, 11, "in"),
   
   /** LEGAL */
   LEGAL("Legal, 8.5\" x 14\"", 8.5, 14, "in"),
   
   /** A0 */
   A0("A0, 841mm x 1189mm", 841, 1189, "mm"),
   
   /** A1 */
   A1("A1, 594mm x 841mm", 594, 841, "mm"),
   
   /** A2 */
   A2("A2, 420mm x 594mm", 420, 594, "mm"),
   
   /** A3 */
   A3("A3, 297mm x 420mm", 297, 420, "mm"),
   
   /** A4 */
   A4("A4, 210mm x 297mm", 210, 297, "mm"),
   
   /** A5 */
   A5("A5, 148mm x 210mm", 148, 210, "mm"),
   
   /** A6 */
   A6("A6, 105mm x 148mm", 105, 148, "mm");
   
   /** A format's description */
   private final String description;
   
   /** A paper width given in the provided units */
   private final double width;
   
   /** A paper width given in the provided units */
   private final double height;
   
   /** Defines units in which document's width and height are expressed. */
   private final String units;
   
   
   /**
    * Constructs this enumeration constant.
    * 
    * @param    description
    *           Its description
    * @param    width
    *           A paper width that has this format
    * @param    height
    *           A paper height that has this format
    * @param    units
    *           Units in which paper width and height are expressed
    */
   private PaperFormat(String description, double width, double height, String units)
   {
      this.description = description;
      this.height = height;
      this.width = width;
      this.units = units;
   }

   /**
    * Returns its description
    * 
    * @return   Its description
    */
   public String getDescription()
   {
      return description;
   }

   /**
    * Returns a width of paper having this format, expressed in the format's units.
    * 
    * @return   A paper width that has this format
    */
   public double getWidth()
   {
      return width;
   }

   /**
    * Returns a height of paper having this format, expressed in the format's units.
    * 
    * @return   A paper height that has this format
    */
   public double getHeight()
   {
      return height;
   }

   /**
    * Returns units in which paper width and height are expressed
    * 
    * @return   Measurement Units
    */
   public String getUnits()
   {
      return units;
   }

}
