/*
** Module   : Directory.java
** Abstract : primary directory API
**
** Copyright (c) 2008-2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 GES 20080409 @37907   Created initial version which has a simple
**                           getter API only. This API is designed to be
**                           simple to use.
** 002 HC  20180813          Added double data type to the directory access interface.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.directory;

/**
 * This interface defines the primary API for directory access. This API is
 * designed with simplicity in mind.  It is currently incomplete, but it
 * does demonstrate a significantly easier mechanism for directory access.
 */
public interface Directory
{
   /** The node ID includes the full path from the root. */
   public static final int ID_ABSOLUTE = 0;
   
   /** The node ID is relative to a default set of search paths. */
   public static final int ID_RELATIVE = 1;
   
   /**
    * Access a data value from the directory as specified by the given
    * absolute or relative node ID.
    *
    * @param    search
    *           Must be one of <code>ID_ABSOLUTE</code> (if the given node
    *           ID includes all paths from the root of the directory tree to
    *           the node being read) or <code>ID_RELATIVE</code> if the given
    *           ID is only a relative portion which should be found via a
    *           search through a standard set of search paths.
    * @param    id
    *           The node ID to retrieve.
    * @param    initial
    *           The initial value to return if a node of the proper type is
    *           not found by the given node ID or if no node is found by the
    *           given node ID.
    *
    * @return   The data value at that node ID or the default initial value.
    */
   public String getString(int search, String id, String initial);
   
   /**
    * Access a data value from the directory as specified by the given
    * absolute or relative node ID.
    *
    * @param    search
    *           Must be one of <code>ID_ABSOLUTE</code> (if the given node
    *           ID includes all paths from the root of the directory tree to
    *           the node being read) or <code>ID_RELATIVE</code> if the given
    *           ID is only a relative portion which should be found via a
    *           search through a standard set of search paths.
    * @param    id
    *           The node ID to retrieve.
    * @param    initial
    *           The initial value to return if a node of the proper type is
    *           not found by the given node ID or if no node is found by the
    *           given node ID.
    *
    * @return   The data value at that node ID or the default initial value.
    */
   public char getChar(int search, String id, char initial);
   
   /**
    * Access a data value from the directory as specified by the given
    * absolute or relative node ID.
    *
    * @param    search
    *           Must be one of <code>ID_ABSOLUTE</code> (if the given node
    *           ID includes all paths from the root of the directory tree to
    *           the node being read) or <code>ID_RELATIVE</code> if the given
    *           ID is only a relative portion which should be found via a
    *           search through a standard set of search paths.
    * @param    id
    *           The node ID to retrieve.
    * @param    initial
    *           The initial value to return if a node of the proper type is
    *           not found by the given node ID or if no node is found by the
    *           given node ID.
    *
    * @return   The data value at that node ID or the default initial value.
    */
   public int getInt(int search, String id, int initial);

   /**
    * Access a data value from the directory as specified by the given
    * absolute or relative node ID.
    *
    * @param    search
    *           Must be one of <code>ID_ABSOLUTE</code> (if the given node
    *           ID includes all paths from the root of the directory tree to
    *           the node being read) or <code>ID_RELATIVE</code> if the given
    *           ID is only a relative portion which should be found via a
    *           search through a standard set of search paths.
    * @param    id
    *           The node ID to retrieve.
    * @param    initial
    *           The initial value to return if a node of the proper type is
    *           not found by the given node ID or if no node is found by the
    *           given node ID.
    *
    * @return   The data value at that node ID or the default initial value.
    */
   public double getDouble(int search, String id, double initial);

   /**
    * Access a data value from the directory as specified by the given
    * absolute or relative node ID.
    *
    * @param    search
    *           Must be one of <code>ID_ABSOLUTE</code> (if the given node
    *           ID includes all paths from the root of the directory tree to
    *           the node being read) or <code>ID_RELATIVE</code> if the given
    *           ID is only a relative portion which should be found via a
    *           search through a standard set of search paths.
    * @param    id
    *           The node ID to retrieve.
    * @param    initial
    *           The initial value to return if a node of the proper type is
    *           not found by the given node ID or if no node is found by the
    *           given node ID.
    *
    * @return   The data value at that node ID or the default initial value.
    */
   public boolean getBoolean(int search, String id, boolean initial);
}