/*
** Module   : MessageTypes.java
** Abstract : defines all valid message types
**
** Copyright (c) 2007-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description------------------------------
** 001 GES 20070111   @31790 Moved the constants here from the Message
**                           class to make it easy for other code to
**                           reference the constants.
** 002 GES 20070115   @31834 Added INIT_CONVERSATION type and changed INIT
**                           to INIT_STANDARD.
** 003 ECF 20071102   @35883 Added INIT_ROUTER type.
** 004 CA  20130822          Fixed the net protocol to support async requests.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.net;

/**
 * Defines all valid message types for the distributed application protocol.
 */
public interface MessageTypes
{
   /** A name to node or node to name look-up request. */
   public static final int ADDRESS_REQUEST = 1;

   /** A test message requesting a protocol-level reply. */
   public static final int ECHO = 2;

   /** A request for the assignment of a node address. */
   public static final int INIT_STANDARD = 3;

   /** A request for the assignment of a node address in conversation mode. */
   public static final int INIT_CONVERSATION = 4;

   /** A synchronous request for processing. */
   public static final int REQUEST_SYNCH = 5;

   /** A message that will be sent to the other side asynchronously. */
   public static final int REQUEST_ASYNCH = 6;
   
   /** A notification that a new router-to-router connection was created. */
   public static final int INIT_ROUTER = 7;

   /** An artificial boundary to ease finding the beginning of reply types. */
   public static final int BEGIN_REPLY = 100;

   /** A name or node look-up response. */
   public static final int ADDRESS_REPLY = 101;

   /** The response to an ECHO request. */
   public static final int ECHO_REPLY = 102;

   /** Contains the dynamically assigned address for the node. */
   public static final int INIT_REPLY = 103;

   /** A successful response to a synchronous request. */
   public static final int REPLY_SYNC = 104;

   /** A successful response to an asynchronous request. */
   public static final int REPLY_ASYNC = 105;

   /** An unsuccessful response or error condition response to a synchronous request. */
   public static final int REPLY_EXCEPTION_SYNC = 106;

   /** An unsuccessful response or error condition response to an asynchronous request. */
   public static final int REPLY_EXCEPTION_ASYNC = 107;

   /** An artificial boundary to ease finding the end of reply types. */
   public static final int END_REPLY = 108;
   
   /** An invalid type. */
   public static final int UNKNOWN = 999;
}