/*
** Module   : ReferenceHint.java
** Abstract : Collection of data that constitutes a hint record for a symbol
**            reference.
**
** Copyright (c) 2004-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 NVS 20050511 ADD   @21157 Created. Initial implementation.
** 002 NVS 20050526 CHG   @21310 javadoc fixes
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.preproc;

/**
 * Keeps information about a symbol reference.
 * <p>
 * @author NVS
 * @version 1.0.0
 */
public class ReferenceHint
{
   /** symbol name */
   private String name = null;

   /** symbol type */
   private String type = null;

   /** symbol value */
   private String value = null;

   /** line number on the output for the first character produced */
   private int line = 0;

   /** column number on the output for the first character produced */
   private int column = 0;

   /**
    * Constructor.
    *
    * @param name
    *        referenced name
    * @param type
    *        symbol type
    * @param value
    *        symbol value
    */
   public ReferenceHint(String name, String type, String value)
   {
      this.name = name;
      this.type = type;
      this.value = value;
   }

   /**
    * Gets reference name.
    *
    * @return  reference name
    */
   public String getName()
   {
      return name;
   }

   /**
    * Gets reference type.
    *
    * @return  reference type
    */
   public String getType()
   {
      return type;
   }

   /**
    * Gets reference value.
    *
    * @return  reference value
    */
   public String getValue()
   {
      return value;
   }

   /**
    * Sets line number.
    *
    * @param   line
    *          line number
    */
   public void setLine(int line)
   {
      this.line = line;
   }

   /**
    * Gets line number.
    *
    * @return line number
    */
   public int getLine()
   {
      return line;
   }

   /**
    * Sets column number.
    *
    * @param   column
    *          column number
    */
   public void setColumn(int column)
   {
      this.column = column;
   }

   /**
    * Gets column number.
    *
    * @return  column number
    */
   public int getColumn()
   {
      return column;
   }
}