/*
** Module   : AmbigFileReportLine.java
** Abstract : POJO with the details of a report line, for a file with ambiguous call-sites.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20170612 First version.
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.report.server;

/**
 * Encapsulates the details of a file with ambiguous call-sites.
 */
public class AmbigFileReportLine
{
   /** The file name. */
   private String filename;
   
   /** The number of ambiguous call-sites. */
   private int num;

   /**
    * Create a new instance with the given data.
    * 
    * @param    filename
    *           The file name.
    * @param    num
    *           The number of ambiguous call-sites.
    */
   public AmbigFileReportLine(String filename, int num)
   {
      this.filename = filename;
      this.num = num;
   }

   /**
    * Get the file name.
    * 
    * @return   The {@link #filename}.
    */
   public String getFilename()
   {
      return filename;
   }

   /**
    * Set the file name.
    * 
    * @param    filename
    *           The file name.
    */
   public void setFilename(String filename)
   {
      this.filename = filename;
   }

   /**
    * Get the number of ambiguous call-sites.
    * 
    * @return   The {@link #num}.
    */
   public int getNum()
   {
      return num;
   }

   /**
    * Set the number of ambiguous call-sites.
    * 
    * @param    num
    *           The number of ambiguous call-sites.
    */
   public void setNum(int num)
   {
      this.num = num;
   }
}
