/*
** Module   : SourceListingApi.java
** Abstract : Implements source file listing services for use by a web client.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 ECF 20170727 First version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.report.server;

import java.sql.*;
import java.util.*;

/**
 * Provides an interface for source file listing services.
 */
public class SourceListingApi
{
   /** AST API base message ID. */
   private static final int MSG_BASE = ReportProtocol.MSG_BASE + 5000;
   
   /** API to retrieve all files with id and type info */
   private static final int MSG_FILE_LISTING = MSG_BASE + 1;
   
   /** Query all source file names with id and type info */
   private static final String sqlQryFileListing =
      "select f.id, f.name, (select group_concat(s.type order by s.type) from source_line s " +
      "join file f1 on f1.id = s.fid and s.line = 0 where f1.id = f.id) as types from file f " +
      "group by f.id, types order by types, f.name";
   
   /** Database services helper object */
   private static final DatabaseService dbs = DatabaseService.get();
   
   /**
    * Default constructor.
    */
   public SourceListingApi()
   {
   }
   
   /**
    * Retrieve a listing of all source files, including their IDs, fully qualified names, and
    * type information.
    *
    * @return  Listing of all source files.
    */
   @WebApi(type = MSG_FILE_LISTING)
   public List<FileListingEntry> fileListing()
   {
      try (Connection conn = dbs.openConnection())
      {
         try (PreparedStatement ps = conn.prepareStatement(sqlQryFileListing))
         {
            List<FileListingEntry> list = new ArrayList<>();
            
            ResultSet rs = ps.executeQuery();
            while (rs.next())
            {
               FileListingEntry entry = new FileListingEntry();
               entry.setId(rs.getLong(1));
               entry.setName(rs.getString(2));
               entry.setTypes(rs.getString(3));
               list.add(entry);
            }
            
            conn.commit();
            
            return list;
         }
         catch (SQLException exc)
         {
            if (conn != null)
            {
               conn.rollback();
            }
            
            throw new RuntimeException(exc);
         }
      }
      catch (SQLException exc)
      {
         throw new RuntimeException(exc);
      }
   }
}
