/*
** Module   : XprTag.java
** Abstract : Currently supported XPR tags definition.
**
** Copyright (c) 2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20180511 Created initial version.
** 002 EVL 20180530 Adding LEFT margin tag.
** 003 EVL 20180605 Adding TOP margin tag.
** 004 EVL 20180625 Adding SAVE, RESTORE and ADJUST tags support.  Adding (O)PORTRAIT and
**                  (O)LANDSCAPE tags support.
** 005 EVL 20180703 Adding /P tag support.
** 006 EVL 20180717 Changed the PREVIEW tag to handle optional tag parameters.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.reporting;

/**
 * The interface defining the string values to detect the XPR tags.
 */
public interface XprTag
{
   /** PREVIEW tag marker. */
   public static final String PREVIEW = "<PREVIEW";
   
   /** MODAL tag marker. */
   public static final String MODAL = "<MODAL";
   
   /** NO_PROGRESS tag marker. */
   public static final String NO_PROGRESS = "</PROGRESS>";

   /** COPIES tag marker. */
   public static final String COPIES = "<COPIES=";

   /** NUM_PAGES tag marker. */
   public static final String NUM_PAGES = "<#PAGES>";
   
   /** Any tag start marker. */
   public static final String START = "<";

   /** Any tag end marker. */
   public static final String END = ">";
   
   /** AT tag marker. */
   public static final String AT = "<AT=";
   
   /** Column setter tag marker. */
   public static final String COL_SET = "<C";

   /** Row setter tag marker. */
   public static final String ROW_SET = "<R";

   /** RECTANGLE tag marker. */
   public static final String RECT = "<RECT";

   /** FILLRECT tag marker. */
   public static final String FILLRECT = "<FILLRECT";

   /** LINE tag marker. */
   public static final String LINE = "<LINE";

   /** BOOKMARK tag marker. */
   public static final String BOOKMARK = "<#";

   /** GOTO BOOKMARK tag marker. */
   public static final String GOTO_BOOKMARK = "<=";
   
   /** FROM tag marker. */
   public static final String FROM = "<FROM>";
   
   /** IMAGE tag marker. */
   public static final String IMAGE = "<IMAGE";
   
   /** FONT tag marker. */
   public static final String FONT = "<F";
   
   /** FONT_SIZE tag marker. */
   public static final String FONT_SIZE = "<P";
   
   /** STOP_FONT_SIZE tag marker. */
   public static final String STOP_FONT_SIZE = "</P";
   
   /** BOLD tag marker. */
   public static final String BOLD = "<B>";
   
   /** NO_BOLD tag marker. */
   public static final String NO_BOLD = "</B>";
   
   /** ITALIC tag marker. */
   public static final String ITALIC = "<I>";
   
   /** NO_ITALIC tag marker. */
   public static final String NO_ITALIC = "</I>";
   
   /** Line width tag marker. */
   public static final String LINE_WIDTH = "<|";
   
   /** FGCOLOR tag marker. */
   public static final String FGCOLOR = "<FGCOLOR=";
   
   /** BGCOLOR tag marker. */
   public static final String BGCOLOR = "<BGCOLOR=";
   
   /** LINECOLOR tag marker. */
   public static final String LINECOLOR = "<LINECOLOR=";
   
   /** UNITS tag marker. */
   public static final String UNITS = "<UNITS=";
   
   /** LEFT tag marker. */
   public static final String LEFT = "<LEFT=";
   
   /** TOP tag marker. */
   public static final String TOP = "<TOP=";
   
   /** BARCODE tag marker. */
   public static final String BARCODE = "<BARCODE";
   
   /** SAVE tag marker. */
   public static final String SAVE = "<SAVE=";
   
   /** RESTORE tag marker. */
   public static final String RESTORE = "<RESTORE=";
   
   /** ADJUST tag marker. */
   public static final String ADJUST = "<ADJUST=";
   
   /** LANDSCAPE tag marker. */
   public static final String LANDSCAPE = "<LANDSCAPE>";
   
   /** PORTRAIT tag marker. */
   public static final String PORTRAIT = "<PORTRAIT>";
   
   /** OLANDSCAPE tag marker. */
   public static final String OLANDSCAPE = "<OLANDSCAPE>";
   
   /** OPORTRAIT tag marker. */
   public static final String OPORTRAIT = "<OPORTRAIT>";
   
   /** Plus character subtag. */
   public static final char CHR_PLUS = '+';

   /** Minus character subtag. */
   public static final char CHR_MINUS = '-';

   /** Sharp character subtag. */
   public static final char CHR_SHARP = '#';
}