/*
** Module   : DataModelTokenTypes.java
** Abstract : Token types for a data model-specific AST.
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 ECF 20050426   @21001 Created initial version.
** 002 ECF 20050608   @21462 Removed unused token types.
** 003 ECF 20050616   @21537 Added relation types: MANY_TO_ONE, ONE_TO_ONE
**                           and ONE_TO_MANY.
** 004 ECF 20050628   @21604 Added COMPONENT type.
** 005 ECF 20050707   @21672 Added tokens needed for initial value
**                           processing. INITIAL, STRING, NUM_LITERAL,
**                           DEC_LITERAL, BOOL_TRUE, BOOL_FALSE,
**                           DATE_LITERAL, UNKNOWN.
** 006 ECF 20050720   @21774 Changed COMPONENT to COMPOSITE.
** 007 ECF 20060401   @25333 Added NULL_LITERAL.
** 010 OM  20121207          Added support for constructing sequence nodes into p2o file.
** 011 GES 20130121          Added DATETIME_TZ_LITERAL.
** 012 OM  20130517          Added DATETIME_LITERAL.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.schema;

/**
 * Token types necessary to represent a data model as an AST.
 */
public interface DataModelTokenTypes
{
   /** Root AST for the data model */
   public static final int DATA_MODEL = 1;
   
   /** Database table */
   public static final int TABLE = 2;
   
   /** Instance variable property of data model class representing a column */
   public static final int PROPERTY = 3;
   
   /** Data model class */
   public static final int CLASS = 4;
   
   /** Unique identifier generator for primary keys */
   public static final int GENERATOR = 5;
   
   /** Primary key */
   public static final int PRIMARY = 6;
   
   /** Foreign key */
   public static final int FOREIGN = 7;
   
   /** Database table index */
   public static final int INDEX = 8;
   
   /** Unique constraint */
   public static final int UNIQUE = 9;
   
   /** Column participating in an index */
   public static final int INDEX_COL = 10;
   
   /** Column participating in a constraint */
   public static final int CONSTR_COL = 11;
   
   /** Many-to-one relationship */
   public static final int MANY_TO_ONE = 12;
   
   /** One-to-many relationship */
   public static final int ONE_TO_MANY = 13;
   
   /** One-to-one relationship */
   public static final int ONE_TO_ONE = 14;
   
   /** Property composite */
   public static final int COMPOSITE = 15;
   
   /** Initial value */
   public static final int INITIAL = 16;
   
   /** String literal */
   public static final int STRING = 17;
   
   /** Integral literal */
   public static final int NUM_LITERAL = 18;
   
   /** Decimal literal */
   public static final int DEC_LITERAL = 19;
   
   /** Boolean true literal */
   public static final int BOOL_TRUE = 20;
   
   /** Boolean false literal */
   public static final int BOOL_FALSE = 21;
   
   /** Date literal */
   public static final int DATE_LITERAL = 22;
   
   /** DATETIME literal */
   public static final int DATETIME_LITERAL = 23;
   
   /** DATETIME-TZ literal */
   public static final int DATETIME_TZ_LITERAL = 24;
   
   /** Unknown value literal */
   public static final int UNKNOWN = 25;
   
   /** Null literal */
   public static final int NULL_LITERAL = 26;
   
   /** Sequence literal */
   public static final int SEQUENCE = 27;
}