/*
** Module   : RightsEditor.java
** Abstract : Interface that custom rights editors implement.
**
** Copyright (c) 2009-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description----------------------
** 001 NVS 20090707  @43089  Created initial version.
** 002 NVS 20090710  @43166  Passing instance of AdminClient that will be
**                           requiredfor using the various utility methods.
**                           Passing rights description to initialize() method.
** 003 NVS 20090723  @43365  Added createDefaultRights() method.
** 004 NVS 20090921  @44086  Extended the edit() method to take the resource 
**                           instance name associated with this instance of 
**                           rights.
** 005 HC  20170612          Changes related to implementation of new GWT-based Admin client.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.security;

import java.io.*;
import java.util.function.*;

/**
 * This interface defines methods that are called by the admin client's
 * Access Control List Management screen when editing ACLs.
 */
public interface RightsEditor
extends Serializable
{
   /**
    * Gives chance to editors to initialize their data structures.
    * <p>
    * This is a one time call for this kind of resource for the life time 
    * of the admin session.
    *
    * @param   description
    *          rights structure description
    */
   void initialize(Description[] description);

   /**
    * Creates an instance of the <code>Rights</code>, which does not take
    * any input and is considered a default starting point in editing.
    *
    * @return  an instance of <code>Rights</code>
    */
   Rights createDefaultRights();
   
   /**
    * Edits the given instance of the <code>Rights</code>. 
    * The implementation should prepare its modal dialog using the initial state
    * from the passed instance of the <code>Rights</code>, show the dialog, 
    * provide event processing until the Save or Cancel action is applied, then
    * hide the dialog and return the appropriate result.
    *
    * @param   name
    *          name of the resource instance being edited
    * @param   exact
    *          nature of the name
    * @param   rights
    *          instance of <code>Rights</code> to be edited
    * @param   submitHandler
    *          Submit handler.
    * @param   cancelHandler
    *          Cancel handler.
    */
   void edit(String name,
             boolean exact,
             Rights rights,
             Consumer<Rights> submitHandler,
             Runnable cancelHandler);
}   