/*
** Module   : SecurityConstants.java
** Abstract : a shared definition of security-related constants
**
** Copyright (c) 2006-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ---------------------------- Description ----------------------------
** 001 GES 20061003   @30106 Created as a shared definition of security-related constants.
** 002 NVS 20090130   @41244 Added AUTH_REQ_PROGRAM constant.
** 003 SVL 20100916          Changed AUTH_MODE_LOWEST in order to include
**                           AUTH_MODE_NONE in the range.
** 004 OM  20131002          Added AUTH_RESULT_SKIP_ENTITY, PKT_SIZE_SKIP_ENTITY constants.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.security;

/**
 * A shared definition of security-related constants.
 */
public interface SecurityConstants
{
   /** Authorization mode: no override. */
   public static final int AUTH_MODE_NONE = 0;

   /** Authorization mode: the lowest value. */
   public static final int AUTH_MODE_LOWEST = 0;

   /** Authorization mode: user ID and password. */
   public static final int AUTH_MODE_IDPW = 1;

   /** Authorization mode: X.509 certificate. */
   public static final int AUTH_MODE_X509 = 2;

   /** Authorization mode: X.509 certificate and userID and password. */
   public static final int AUTH_MODE_X509_IDPW = 3;

   /** Authorization mode: custom. */
   public static final int AUTH_MODE_CUSTOM = 4;

   /** Authorization mode: the highest value. */
   public static final int AUTH_MODE_HIGHEST = 5;

   /** Authorization request type: USER. */
   public static final int AUTH_REQ_USER = 0;

   /** Authorization request type: PROCESS. */
   public static final int AUTH_REQ_PROCESS = 1;

   /** Authorization request type: PROGRAMMATIC */
   public static final int AUTH_REQ_PROGRAM = 2;

   /** Authorization result: no result. */
   public static final int AUTH_RESULT_NONE = -1;

   /** Authorization result: success. */
   public static final int AUTH_RESULT_SUCCESS = 0;

   /** Authorization result: unspecified failure. */
   public static final int AUTH_RESULT_UNSPECIFIED_FAILURE = 1;
   
   /** Authorization result: invalid user ID. */
   public static final int AUTH_RESULT_INVALID_USERID = 2;

   /** Authorization result: invalid password. */
   public static final int AUTH_RESULT_INVALID_PASSWORD = 3;

   /** Authorization result: insufficient account rights to logon. */
   public static final int AUTH_RESULT_INSUFFICIENT_RIGHTS = 4;

   /** Cancel current authorization. */
   public static final int AUTH_RESULT_SKIP_TO_NEXT = 5;

   /** Authorization action: authorization has been successfully completed. */
   public static final int AUTH_ACTION_DONE = 0;
   
   /** Authorization action: authorization should continue. */
   public static final int AUTH_ACTION_CONTINUE = 1;
   
   /** Authorization action: abort further processing immediately. */
   public static final int AUTH_ACTION_ABORT = 2;

   /** Authorization action: retry authentication. */
   public static final int AUTH_ACTION_RETRY = 3;

   /** The (symbolic) size of package represented by AUTH_RESULT_SKIP_TO_NEXT. */
   public static final int PKT_SIZE_SKIP_TO_NEXT = -1;
}