/*
** Module   : TokenDataWrapper.java
** Abstract : interface to store/access token related data in an object
**            wrapper
**
** Copyright (c) 2005-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 GES 20050415 ADD   @20720 First version with a simple bean-like 
**                               approach to store/access a name, an integer
**                               token type and a temporary index.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.uast;

/**
 * A simple wrapper interface to access basic token data in a wrappered
 * object.
 *
 * @author    GES
 * @version   1.0.0
 */
public interface TokenDataWrapper
{
   /**
    * Returns the full name of the object.
    *
    * @return   The full name of the object.
    */
   public String getName();
   
   /**
    * Sets the full name of the object.
    *
    * @param    name
    *           The full name of the object.
    */
   public void setName(String name);
   
   /**
    * Returns the parser-defined token type of the object.
    *
    * @return   The token type of the object.
    */
   public int getTokenType();
   
   /**
    * Sets the parser-defined token type of this object.
    *
    * @param    tokenType
    *           The token type of the object.
    */
   public void setTokenType(int tokenType);
   
   /**
    * Returns the temporary index that identifies the original AST node that
    * defined the object.
    *
    * @return   The temporary index.
    */
   public int getTempIndex();
   
   /**
    * Sets the temporary index that identifies the original AST node that
    * defined the object.
    *
    * @param    tempIdx
    *           The temporary index.
    */
   public void setTempIndex(int tempIdx);   
}