/*
** Module   : BrowseColumnInterface.java
** Abstract : Define all BROWSE column widget attributes and methods available via a handle
*             instance.
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 SVL 20141106 Created initial version.
** 002 SVL 20151013 Changed setBgColor signature.
** 003 SVL 20170120 Added COLUMN-FGCOLOR, SORT-ASCENDING and SORT-NUMBER attributes.
** 004 SVL 20170813 setBg(Fg)Color renamed to setColumnBg(Fg)Color, added setColumnReadOnly,
**                  bufferField.
** 005 SVL 20180614 Added setHyperlinkEvent.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Define all BROWSE column widget attributes and methods which can be accessed via a handle
 * instance.
 */
public interface BrowseColumnInterface
{
   /**
    * Sets the COLUMN-BGCOLOR writable attribute.
    *
    * @param    param
    *           The new value for the COLUMN-BGCOLOR attribute.
    */
   @LegacyAttribute(name = "COLUMN-BGCOLOR", setter = true)
   public void setColumnBgColor(NumberType param);

   /**
    * Sets the COLUMN-BGCOLOR writable attribute.
    *
    * @param    param
    *           The new value for the COLUMN-BGCOLOR attribute.
    */
   @LegacyAttribute(name = "COLUMN-BGCOLOR", setter = true)
   public void setColumnBgColor(long param);

   /**
    * Gets the COLUMN-BGCOLOR writable attribute.
    *
    * @return   The current value of the COLUMN-BGCOLOR attribute.
    */
   @LegacyAttribute(name = "COLUMN-BGCOLOR")
   public integer getColumnBgColor();

   /**
    * Sets the COLUMN-FGCOLOR writable attribute.
    *
    * @param    param
    *           The new value for the COLUMN-FGCOLOR attribute.
    */
   @LegacyAttribute(name = "COLUMN-FGCOLOR", setter = true)
   public void setColumnFgColor(NumberType param);

   /**
    * Sets the COLUMN-FGCOLOR writable attribute.
    *
    * @param    param
    *           The new value for the COLUMN-FGCOLOR attribute.
    */
   @LegacyAttribute(name = "COLUMN-FGCOLOR", setter = true)
   public void setColumnFgColor(long param);

   /**
    * Gets the COLUMN-FGCOLOR writable attribute.
    *
    * @return   The current value of the COLUMN-FGCOLOR attribute.
    */
   @LegacyAttribute(name = "COLUMN-FGCOLOR")
   public integer getColumnFgColor();

   /**
    * Implements the NEXT-COLUMN attribute getter.<p>
    *
    * @return the widget handle of the next sibling, in physical order, of the current browse
    *         column whether or not the column is visible.
    */
   @LegacyAttribute(name = "NEXT-COLUMN")
   public handle getNextColumn();

   /**
    * Sets the SORT-ASCENDING attribute, which defines the state of the sort arrow in the column
    * header.
    *
    * @param    ascending
    *           The new value for the SORT-ASCENDING attribute. Unknown value to hide the arrow,
    *           <code>true</code> for ascending sorting (arrow up). <code>false</code> for
    *           descending sorting (arrow down).
    */
   @LegacyAttribute(name = "SORT-ASCENDING", setter = true)
   public void setSortAscending(logical ascending);

   /**
    * Sets the SORT-ASCENDING attribute, which defines the state of the sort arrow in the column
    * header.
    *
    * @param    ascending
    *           The new value for the SORT-ASCENDING attribute. Unknown value to hide the arrow,
    *           <code>true</code> for ascending sorting (arrow up). <code>false</code> for
    *           descending sorting (arrow down).
    */
   @LegacyAttribute(name = "SORT-ASCENDING", setter = true)
   public void setSortAscending(boolean ascending);

   /**
    * Gets the SORT-ASCENDING attribute, which defines the state of the sort arrow in the column
    * header.
    *
    * @return   The current value of the SORT-ASCENDING attribute. Unknown value if the arrow is
    *           hidden, <code>true</code> for ascending sorting (arrow up). <code>false</code> for
    *           descending sorting (arrow down).
    */
   @LegacyAttribute(name = "SORT-ASCENDING")
   public logical isSortAscending();

   /**
    * Sets the SORT-NUMBER attribute, which indicates the sort-level number for the browse column.
    * The number is displayed next to the sorting arrow.
    *
    * @param    sortNumber
    *           The new value for the SORT-NUMBER attribute. Unknown or <code>0</code> to hide the
    *           sort-level number, otherwise numbers from <code>1</code> to <code>9</code> are
    *           allowed.
    */
   @LegacyAttribute(name = "SORT-NUMBER", setter = true)
   public void setSortNumber(int64 sortNumber);

   /**
    * Sets the SORT-NUMBER attribute, which indicates the sort-level number for the browse column.
    * The number is displayed next to the sorting arrow.
    *
    * @param    sortNumber
    *           The new value for the SORT-NUMBER attribute. Unknown or <code>0</code> to hide the
    *           sort-level number, otherwise numbers from <code>1</code> to <code>9</code> are
    *           allowed.
    */
   @LegacyAttribute(name = "SORT-NUMBER", setter = true)
   public void setSortNumber(int sortNumber);

   /**
    * Get the SORT-NUMBER attribute, which indicates the sort-level number for the browse column.
    * The number is displayed next to the sorting arrow.
    *
    * @return   The current value of the SORT-NUMBER attribute.
    */
   @LegacyAttribute(name = "SORT-NUMBER")
   public integer getSortNumber();

   /**
    * Sets the COLUMN-READ-ONLY attribute, which indicates whether you can tab to a browse column
    * but not edit it.
    *
    * @param    columnReadOnly
    *           The new value for the COLUMN-READ-ONLY attribute. <code>true</code> if you can tab
    *           to a browse column but not edit it.
    */
   @LegacyAttribute(name = "COLUMN-READ-ONLY", setter = true)
   public void setColumnReadOnly(logical columnReadOnly);

   /**
    * Sets the COLUMN-READ-ONLY attribute, which indicates whether you can tab to a browse column
    * but not edit it.
    *
    * @param    columnReadOnly
    *           The new value for the COLUMN-READ-ONLY attribute. <code>true</code> if you can tab
    *           to a browse column but not edit it.
    */
   @LegacyAttribute(name = "COLUMN-READ-ONLY", setter = true)
   public void setColumnReadOnly(boolean columnReadOnly);

   /**
    * Get the COLUMN-READ-ONLY attribute, which indicates whether you can tab to a browse column
    * but not edit it.
    *
    * @return   The current value of the COLUMN-READ-ONLY attribute.
    */
   @LegacyAttribute(name = "COLUMN-READ-ONLY")
   public logical getColumnReadOnly();

   /**
    * Get the handle of the browse column's BUFFER-FIELD.
    *
    * @return  the handle of the browse column's buffer-field.
    */
   @LegacyMethod(name = "BUFFER-FIELD")
   public handle bufferField();

   /**
    * Sets the HYPERLINK attribute (FWD extension), which indicates the name of the event which
    * is raised when a hyperlink from this column is clicked.
    *
    * @param    eventName
    *           The name of the event which is raised when a hyperlink from this column is
    *           clicked. <code>null</code> do disable hyperlinks for this column.
    */
   @LegacyAttribute(name = "HYPERLINK", setter = true)
   void setHyperlinkEvent(String eventName);

   /**
    * Sets the HYPERLINK attribute (FWD extension), which indicates the name of the event which
    * is raised when a hyperlink from this column is clicked.
    *
    * @param    eventName
    *           The name of the event which is raised when a hyperlink from this column is
    *           clicked. <code>null</code> do disable hyperlinks for this column.
    */
   @LegacyAttribute(name = "HYPERLINK", setter = true)
   void setHyperlinkEvent(character eventName);

   /**
    * Returns the HYPERLINK attribute (FWD extension), which indicates the name of the event which
    * is raised when a hyperlink from this column is clicked.
    *
    * @return the name of the event which is raised when a hyperlink from this column is clicked.
    *         <code>null</code> if this is not a hyperlinked column.
    */
   @LegacyAttribute(name = "HYPERLINK")
   character getHyperlinkEvent();
}