/*
** Module   : ColorAttribute.java
** Abstract : Declares the supported terminal text attributes.
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------Description-----------------
** 001 HC  20140119 Created initial version.
** 002 MAG 20140211 Source code formating.
** 003 CA  20171009 Added valueOf.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

/**
 * Set of supported console text attributes with their respective values. 
 * The values allow the attributes to be stored as an integral bit set. 
 * See {@link Color}.
 */
public enum ColorAttribute 
{
   NORMAL(0x00), 
   BLINK(0x01), 
   BOLD(0x02), 
   UNDERLINE(0x04), 
   REVERSE(0x08);

   /** the attribute value */
   private int value;

   /**
    * The constructor.
    * 
    * @param   value
    *          the attribute value 
    */
   private ColorAttribute(int value) 
   {
       this.value = value;
   }
   
   /**
    * Resolve the {@link ColorAttribute} instance for the specified {@link #value}.
    * 
    * @param    value
    *           The value to be resolved.
    *           
    * @return   The {@link ColorAttribute} enum entry or {@link #NORMAL} if not a valid value.
    */
   public static ColorAttribute valueOf(int value)
   {
      switch (value)
      {
         case 0x01:
            return BLINK;
         case 0x02:
            return BOLD;
         case 0x04:
            return UNDERLINE;
         case 0x08:
            return REVERSE;
      }
      
      return NORMAL;
   }

   /**
    * Returns the attribute value.
    * 
    * @return   See above.
    */
   public int getValue() 
   {
       return value;
   }
}