/*
** Module   : FieldGroupInterface.java
** Abstract : Defines FIELD-GROUP widget attributes and methods available via a handle instance.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 SBI 20171003 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Defines FIELD-GROUP widget attributes and methods available via a handle instance.
 */
public interface FieldGroupInterface
{
   /**
    * Getting the FIRST-TAB-ITEM attribute.
    * 
    * @return  The widget handle that is the first in the frame tab order
    */
   @LegacyAttribute(name = "FIRST-TAB-ITEM")
   public handle getFirstTabItem();
   
   /**
    * Setting the FIRST-TAB-ITEM attribute.
    * 
    * @param   wHandle
    *          The handle of widget in this field-group to assign as first in the tab order.
    */
   @LegacyAttribute(name = "FIRST-TAB-ITEM", setter = true)
   public void setFirstTabItem(handle wHandle);
   
   /**
    * Getting the LAST-TAB-ITEM attribute.
    * 
    * @return  The widget handle that is the last in the frame tab order
    */
   @LegacyAttribute(name = "LAST-TAB-ITEM")
   public handle getLastTabItem();
   
   /**
    * Setting the LAST-TAB-ITEM attribute.
    * 
    * @param   wHandle
    *          The handle of widget in this field-group to assign as last in the tab order.
    */
   @LegacyAttribute(name = "LAST-TAB-ITEM", setter = true)
   public void setLastTabItem(handle wHandle);
   
   /**
    * Get the NUM-TABS attribute, which indicates the number of tab widgets inside this
    * field-group container.
    *
    * @return   The current value of the NUM-TABS attribute.
    */
   @LegacyAttribute(name = "NUM-TABS")
   public integer getNumTabs();

   /**
    * Get the handle of the tab item by the given tab position.
    * 
    * @param    tabPosition
    *           The tab position
    * 
    * @return   The handle of the tab item by the given tab position.
    */
   @LegacyMethod(name = "GET-TAB-ITEM")
   public handle getTabItem(integer tabPosition);
   
   /**
    * Get the handle of the tab item by the given tab position.
    * 
    * @param    tabPosition
    *           The tab position
    * 
    * @return   The handle of the tab item by the given tab position.
    */
   @LegacyMethod(name = "GET-TAB-ITEM")
   public handle getTabItem(int tabPosition);
}