/*
** Module   : RectangleInterface.java
** Abstract : provides an API for rectangle-specific attributes and methods 
**
** Copyright (c) 2016-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 GES 20160909 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Provides an API for rectangle-specific attributes and methods.
 */
public interface RectangleInterface
{
   /**
    * Get EDGE-CHARS attribute.
    *          
    * @return  The value of the EDGE-CHARS attribute.
    */
   @LegacyAttribute(name = "EDGE-CHARS")
   public decimal getEdgeChars();

   /**
    * Get EDGE-PIXELS attribute.
    * 
    * @return  The value of the EDGE-PIXELS attribute.
    */
   @LegacyAttribute(name = "EDGE-PIXELS")
   public integer getEdgePixels();

   /**
    * Get FILLED attribute.
    * 
    * @return  The value of the FILLED attribute.
    */
   @LegacyAttribute(name = "FILLED")
   public logical getFilled();
   
   /**
    * Get GRAPHIC-EDGE option.
    * 
    * @return  The value of the GRAPHIC-EDGE attribute.
    */
   @LegacyAttribute(name = "GRAPHIC-EDGE")
   public logical getGraphicEdge();

   /**
    * Get GROUP-BOX attribute.
    * 
    * @return  The value of the GROUP-BOX attribute.
    */
   @LegacyAttribute(name = "GROUP-BOX")
   public logical getGroupBox();
   
   /**
    * Get ROUNDED attribute.
    * 
    * @return  The value of the ROUNDED attribute.
    */
   @LegacyAttribute(name = "ROUNDED")
   public logical getRounded();
   
   /**
    * Set FILLED attribute or NO-FILL option.
    * 
    * @param   flag
    *          New value for the FILLED attribute.
    */
   @LegacyAttribute(name = "FILLED", setter = true)
   public void setFilled(boolean flag);

   /**
    * Set FILLED attribute NO-FILL option.
    * 
    * @param   flag
    *          New value for the FILLED attribute.
    */
   @LegacyAttribute(name = "FILLED", setter = true)
   public void setFilled(logical flag);

   /**
    * Set EDGE-PIXELS option.
    * 
    * @param   pixels
    *          Option value.
    */
   @LegacyAttribute(name = "EDGE-PIXELS", setter = true)
   public void setEdgePixels(int pixels);

   /**
    * Set EDGE-PIXELS option.
    * 
    * @param   pixels
    *          Option value.
    */
   @LegacyAttribute(name = "EDGE-PIXELS", setter = true)
   public void setEdgePixels(integer pixels);

   /**
    * Set EDGE-CHARS option.
    *
    * @param   chars
    *          Option value.
    */
   @LegacyAttribute(name = "EDGE-CHARS", setter = true)
   public void setEdgeChars(double chars);

   /**
    * Set EDGE-CHARS option.
    * 
    * @param   chars
    *          Option value.
    */
   @LegacyAttribute(name = "EDGE-CHARS", setter = true)
   public void setEdgeChars(NumberType chars);

   /**
    * Set EDGE-CHARS option.
    * 
    * @param   chars
    *          Option value.
    */
   @LegacyAttribute(name = "EDGE-CHARS", setter = true)
   public void setEdgeChars(int chars);

   /**
    * Set GRAPHIC-EDGE option.
    * 
    * @param   flag
    *          New value for the option.
    */
   @LegacyAttribute(name = "GRAPHIC-EDGE", setter = true)
   public void setGraphicEdge(boolean flag);

   /**
    * Set GRAPHIC-EDGE option.
    * 
    * @param   flag
    *          New value for the option.
    */
   @LegacyAttribute(name = "GRAPHIC-EDGE", setter = true)
   public void setGraphicEdge(logical flag);

   /**
    * Set GROUP-BOX attribute.
    * 
    * @param   flag
    *          New value for the GROUP-BOX attribute.
    */
   @LegacyAttribute(name = "GROUP-BOX", setter = true)
   public void setGroupBox(boolean flag);

   /**
    * Set GROUP-BOX attribute.
    * 
    * @param   flag
    *          New value for the GROUP-BOX attribute.
    */
   @LegacyAttribute(name = "GROUP-BOX", setter = true)
   public void setGroupBox(logical flag);

   /**
    * Set ROUNDED attribute.
    * 
    * @param   flag
    *          New value for the ROUNDED attribute.
    */
   @LegacyAttribute(name = "ROUNDED", setter = true)
   public void setRounded(boolean flag);

   /**
    * Set ROUNDED attribute.
    * 
    * @param   flag
    *          New value for the ROUNDED attribute.
    */
   @LegacyAttribute(name = "ROUNDED", setter = true)
   public void setRounded(logical flag);
}