/*
** Module   : SelectionListBodyImpl.java
** Abstract :
**
** Copyright (c) 2011-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 SIY 20111202 Extracted from SelectionListBody.
** 002 SVL 20120821 Changed c'tor signature in order to match the one used by ChuiWidgetFactory.
** 003 GES 20140627 Changed color attribute method names to match naming standards.
** 004 CA  20140805 Do not allow ChUI impl classes access to APIs outside of ChuiOutputManager.
** 005 HC  20140911 GUI coordinate support - data types refactoring.
** 006 CA  20140926 Added shared widget configuration support, to allow GUI/ChUI concrete
**                  implementation for the same widget ID.  Refactored the widget configuration
**                  classes: all fields were made public; these classes need to be as dumb as
**                  possible, all logic related to setting/getting a certain field should be at
**                  the widget or at the caller. Refs #2254
** 007 HC  20150323 Removed business logic from config classes.
** 008 HC  20151117 Fixed broken ChUI SELECTION-LIST scrolling and the selection box width.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.chui;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.chui.driver.*;

/**
 * CHUI-specific part of the SelectionListBody.
 */
public class SelectionListBodyImpl
extends SelectionListBody<ChuiOutputManager>
{
   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public SelectionListBodyImpl(WidgetId id, SelectionListConfig cfg)
   {
      super(id, cfg);
   }

   /**
    * Request focus.
    */
   @Override
   public void requestFocus()
   {
      super.requestFocus();

      screen().at(screenLocation().translate(0, currentRow()));
   }

   /**
    * Get widget width.
    * 
    * @return  widget width.
    */
   @Override
   public double width()
   {
      return parent().width();
   }
   
   /**
    * Draws this component. Overrides draw() in JList.
    */
   @Override
   public void draw()
   {
      Color dc = UiUtils.getDColor(this);
      Color pf = config.pfcolor;

      /* Get the absolute origin of this component.
      */
      Point origin = screenLocation();

      StringBuilder blanks = new StringBuilder();
      int maxLength = (int) width();
      
      for (int j = 0; j < maxLength; j++)
         blanks.append(' ');
      
      // obtain the initialization status
      boolean uninit = getState() == ScreenBuffer.UNINITIALIZED;
      boolean enabled = isEnabled();
      int modelRows = model().size();
      
      for (int i = rowTop; i < rowTop + visibleRows(); i++, origin.y++)
      {
         screen().at(origin);

         Color color = dc;

         if (widgetFocused && enabled && i == currentRow())
            color = pf;

         StringBuilder buf = new StringBuilder();
         
         if (i < modelRows)
         {
            if (selected(i) && !uninit)
               buf.append("> ");
            else
               buf.append("  ");
            
            buf.append(getItemText(i));
         }
         
         buf.append(blanks).setLength(maxLength);
         screen().append(buf.toString(), color);
      }
   }
}