/*
** Module   : Insets.java
** Abstract : Widget insets object.
**
** Copyright (c) 2010-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 SIY 20100629          Created initial version
** 002 SIY 20100804          Changes in imports, some refactorings.
** 003 HC  20140911          GUI coordinate support - data types refactoring.
** 004 HC  20141021          Implemented window move/position support.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client;

import com.goldencode.p2j.ui.Coordinate;

/**
 * Insets object.
 * The coordinate values are expressed in character units.
 */
public class Insets
{
   /** Constant object which represents zero insets. */
   public static final Insets ZERO_INSETS = new Insets(0,0,0,0);

   /** Insets at top side. */
   public double top;

   /** Insets at left side. */
   public double left;

   /** Insets at bottom side. */
   public double bottom;

   /** Insets at right side. */
   public double right;
   
   /**
    * Constructor.
    * 
    * @param   top
    *          Top side insets.
    * @param   left
    *          Left side insets.
    * @param   bottom
    *          Bottom side insets.
    * @param   right
    *          Right side insets.
    */
   public Insets(double top, double left, double bottom, double right)
   {
      this.top = Coordinate.scale(top);
      this.left = Coordinate.scale(left);
      this.bottom = Coordinate.scale(bottom);
      this.right = Coordinate.scale(right);
   }

   /**
    * Get sum of left and right deltas.
    * 
    * @return  sum of left and right deltas.
    */
   public double widthDelta()
   {
      return Coordinate.scale(left + right);
   }

   /**
    * Get sum of top and bottom deltas.
    * 
    * @return  sum of top and bottom deltas.
    */
   public double heightDelta()
   {
      return Coordinate.scale(top + bottom);
   }

   /**
    * Get starting point of inner area.
    *  
    * @return  starting point of inner area.
    */
   public Point asPoint()
   {
      return new Point(left, top);      
   }
}