/*
** Module   : MousePtrWrapper.java
** Abstract : Mouse cursor helper.
**
** Copyright (c) 2015-2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------------------Description--------------------------------------
** 001 IAS 20151224 Created initial version.
** 002 HC  20180525 Fixed mouse pointer loading.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/
package com.goldencode.p2j.ui.client;

import java.awt.Cursor;
import java.util.*;

import com.goldencode.p2j.ui.client.gui.driver.CursorType;

/**
 * Mouse cursor helper.
 */
public interface MousePtrWrapper
{
   /** Helpers for pre-defined cursors */
   public static final MousePtrWrapper DEFAULT = PreDefinedMousePtrWrapper.DEFAULT;
   
   public static final MousePtrWrapper TEXT = PreDefinedMousePtrWrapper.TEXT;
   
   public static final MousePtrWrapper N_RESIZE = PreDefinedMousePtrWrapper.N_RESIZE;
   
   public static final MousePtrWrapper NE_RESIZE = PreDefinedMousePtrWrapper.NE_RESIZE;
   
   public static final MousePtrWrapper E_RESIZE = PreDefinedMousePtrWrapper.E_RESIZE;
   
   public static final MousePtrWrapper SE_RESIZE = PreDefinedMousePtrWrapper.SE_RESIZE;
   
   public static final MousePtrWrapper S_RESIZE = PreDefinedMousePtrWrapper.S_RESIZE;
   
   public static final MousePtrWrapper SW_RESIZE = PreDefinedMousePtrWrapper.SW_RESIZE;
   
   public static final MousePtrWrapper W_RESIZE = PreDefinedMousePtrWrapper.W_RESIZE;
   
   public static final MousePtrWrapper NW_RESIZE = PreDefinedMousePtrWrapper.NW_RESIZE;
   
   public static final MousePtrWrapper CROSS = PreDefinedMousePtrWrapper.CROSS;
   
   public static final MousePtrWrapper WAIT = PreDefinedMousePtrWrapper.WAIT;
   
   public static final MousePtrWrapper MOVE = PreDefinedMousePtrWrapper.MOVE;
   
   public static final MousePtrWrapper HAND = PreDefinedMousePtrWrapper.HAND;

   /** Helpers for pre-defined cursors wrapped with Optional */
   public static final Optional<MousePtrWrapper> DEFAULT_PTR = Optional.of(DEFAULT);

   public static final Optional<MousePtrWrapper> TEXT_PTR = Optional.of(TEXT);
   
   public static final Optional<MousePtrWrapper> N_RESIZE_PTR = Optional.of(N_RESIZE);
   
   public static final Optional<MousePtrWrapper> NE_RESIZE_PTR = Optional.of(NE_RESIZE);
   
   public static final Optional<MousePtrWrapper> E_RESIZE_PTR = Optional.of(E_RESIZE);
   
   public static final Optional<MousePtrWrapper> SE_RESIZE_PTR = Optional.of(SE_RESIZE);
   
   public static final Optional<MousePtrWrapper> S_RESIZE_PTR = Optional.of(S_RESIZE);
   
   public static final Optional<MousePtrWrapper> SW_RESIZE_PTR = Optional.of(SW_RESIZE);
   
   public static final Optional<MousePtrWrapper> W_RESIZE_PTR = Optional.of(W_RESIZE);
   
   public static final Optional<MousePtrWrapper> NW_RESIZE_PTR = Optional.of(NW_RESIZE);
   
   public static final Optional<MousePtrWrapper> CROSS_PTR = Optional.of(CROSS);
   
   public static final Optional<MousePtrWrapper> WAIT_PTR = Optional.of(WAIT);
   
   public static final Optional<MousePtrWrapper> MOVE_PTR = Optional.of(MOVE);
   
   public static final Optional<MousePtrWrapper> HAND_PTR = Optional.of(HAND);
   
   /**
    * Check if the cursor is predefined
    * 
    * @return <code>true</code> if the cursor is predefined
    */
   public abstract boolean isPredefined();
   
   /**
    * Get the cursor style
    * 
    * @return the cursor style
    */
   public default int style()
   {
      throw new UnsupportedOperationException();
   }
   
   /**
    * Get the cursor type
    * 
    * @return the cursor type
    */
   public default CursorType type()
   {
      throw new UnsupportedOperationException();
   }

   /**
    * Get the AWT cursor
    * 
    * @return the cursor style
    */
   public default Cursor cursor()
   {
      throw new UnsupportedOperationException();
   }

   
   /**
    * Get the cursor mime type
    * 
    * @return the mime type
    */
   public default String mimeType()
   {
      throw new UnsupportedOperationException();
   }
   
   /**
    * Get the cursor data
    * 
    * @return the cursor data
    */
   public default byte[] data()
   {
      throw new UnsupportedOperationException();
   }
   
   /**
    * Create non-animated cursor helper
    * 
    * @param c
    *        cursor data
    * @return
    *        cursor helper
    */
   public static MousePtrWrapper createStaticSwingInstance(Cursor c)
   {
      return new StaticMousePtrWrapper(c);
   }
   
   /**
    * Create non-animated web cursor helper
    * 
    * @param data
    *        cursor data
    * @param mimeType
    *        cursor mime type
    * @return
    *        cursor helper
    */
   public static MousePtrWrapper createStaticWebInstance(byte[] data, String mimeType)
   {
      return new BinaryMousePtrWrapper(data, mimeType);
   }
   
   /**
    * Create pre-defined cursor helper
    * 
    * @param c
    *        cursor type
    * @return
    *        cursor helper
    */
   public static MousePtrWrapper createPreDefinedInstance(CursorType c)
   {
      return PreDefinedMousePtrWrapper.valueOf(c);
   }

   /**
    * Create animated cursor helper
    * 
    * @param c
    *        cursor data
    * @return
    *        cursor helper
    */
   public static MousePtrWrapper createAnimatedInstance(Object c)
   {
      throw new UnsupportedOperationException("Animated mouse cursors are not supported yet");
   }

   /** Helpers for non-animated cursors */
   public static class StaticMousePtrWrapper 
   implements  MousePtrWrapper
   {
      /** cursor data */
      private final Cursor cursor;

      /**
       * Constructor
       * 
       * @param cursor
       *        cursor data
       */
      public StaticMousePtrWrapper(Cursor cursor)
      {
         super();
         this.cursor = cursor;
      }

     /**
      * Check if the cursor is predefined
      * 
      * @return <code>true</code> if the cursor is predefined
      */
      @Override
      public boolean isPredefined()
      {
         return false;
      }

      /**
       * Get the AWT cursor
       * 
       * @return the cursor style
       */
      public Cursor cursor()
      {
         return cursor;
      }

   }

   /** Helpers for non-animated web cursors */
   public static class BinaryMousePtrWrapper 
   implements  MousePtrWrapper
   {
      /** cursor data */
      private final byte[] data;
      private final String mimeType;  

      /**
       * Constructor
       * 
       * @param data
       *        cursor data
       */
      public BinaryMousePtrWrapper(byte[] data, String mimeType)
      {
         super();
         this.data = data;
         this.mimeType = mimeType;
      }

     /**
      * Check if the cursor is predefined
      * 
      * @return <code>true</code> if the cursor is predefined
      */
      @Override
      public boolean isPredefined()
      {
         return false;
      }

      /**
       * Get the cursor mime type
       * 
       * @return the mime type
       */
      @Override
      public String mimeType()
      {
         return mimeType;
      }

      /**
       * Get the cursor data
       * 
       * @return the cursor data
       */
      @Override
      public byte[] data()
      {
         return data;
      }
   }

   /** Helpers for pre-defined cursors */
   public static enum PreDefinedMousePtrWrapper 
   implements  MousePtrWrapper
   {
      DEFAULT(CursorType.DEFAULT),
      
      TEXT(CursorType.TEXT),
      
      N_RESIZE(CursorType.N_RESIZE),
      
      NE_RESIZE(CursorType.NE_RESIZE),
      
      E_RESIZE(CursorType.E_RESIZE),
      
      SE_RESIZE(CursorType.SE_RESIZE),
      
      S_RESIZE(CursorType.S_RESIZE),
      
      SW_RESIZE(CursorType.SW_RESIZE),
      
      W_RESIZE(CursorType.W_RESIZE),
      
      NW_RESIZE(CursorType.NW_RESIZE),
      
      CROSS(CursorType.CROSS),
      
      WAIT(CursorType.WAIT),
      
      MOVE(CursorType.MOVE),
      
      HAND(CursorType.HAND);
      
      /** cursor type */
      private final CursorType type;

      /**
       * Constructor
       * 
       * @param type
       *        cursor type
       */
      private PreDefinedMousePtrWrapper(CursorType type)
      {
         this.type = type;
      }
      
      /**
       * Retrieve the helper for a cursor type
       * 
       * @param type
       *        cursor type
       *        
       * @return
       *        helper or <code>null</code> if the type is unknown
       */
      public static PreDefinedMousePtrWrapper valueOf(CursorType type)
      {
         return Arrays.stream(values()).filter(p -> p.type == type).findFirst().orElse(null);
      }

      /**
       * Get the cursor type
       * 
       * @return the cursor type
       */
      public CursorType type()
      {
         return type; 
      }

      /**
       * Get the String representation of the helper
       *
       * @return the the String representation of the helper
       */
      @Override
      public String toString()
      {
         return type.toString();
      }

      /**
       * Check if the cursor is predefined
       * 
       * @return <code>true</code> if the cursor is predefined
       */
      @Override
      public boolean isPredefined()
      {
         return true;
      }

      /**
       * Get the cursor style
       * 
       * @return the cursor style
       */
      @Override
      public int style()
      {
         return type.awtCursor;
      }
   }
}