/*
** Module   : ChuiWindowsKeyboard.java
** Abstract : Keyboard configuration for Windows ChUI clients. 
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20141029 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.chui;

import java.util.*;

import com.goldencode.p2j.ui.*;

/**
 * This class contains the configurations for a Keyboard compatible with the Windows ChUI clients.
 */
public class ChuiWindowsKeyboard
extends Keyboard
{
   /**
    * Default c'tor.  Sets the {@link Keyboard#bit10} label to <code>ALT</code>.
    */
   public ChuiWindowsKeyboard()
   {
      super("ALT");
   }
   
   /**
    * Initialize this keyboard.
    */
   @Override
   public void init()
   {
      super.init();

      basicKeys[13] = "ENTER";
      basicKeys[509] = "SHIFT-TAB";
      basicKeys[512] = "DELETE";
      basicKeys[525] = "ENTER";
      basicKeys[610] = "MIDDLE-MOUSE-UP";
      basicKeys[618] = "MIDDLE-MOUSE-DOWN";
      basicKeys[626] = "MIDDLE-MOUSE-CLICK";
      basicKeys[650] = "MIDDLE-MOUSE-DBLCLICK";
      
      // differences for multi-byte keys
      extendedKeys.put(1021, "SHIFT-SHIFT-TAB");
      extendedKeys.put(1024, "ALT");
      extendedKeys.put(1033, "ALT-CTRL-I");
      extendedKeys.put(1037, "ALT-ENTER");
      extendedKeys.put(1533, "ALT-SHIFT-TAB");
      extendedKeys.put(1536, "ALT-DELETE");
      extendedKeys.put(1545, "SHIFT-ALT-CTRL-I");
      extendedKeys.put(1549, "SHIFT-ALT-ENTER");
      extendedKeys.put(2045, "SHIFT-ALT-SHIFT-TAB");
      extendedKeys.put(2061, "CTRL-ENTER");
      extendedKeys.put(2557, "CTRL-SHIFT-TAB");
      extendedKeys.put(2560, "CTRL-DELETE");
      extendedKeys.put(2573, "CTRL-SHIFT-ENTER");
      extendedKeys.put(3069, "CTRL-SHIFT-SHIFT-TAB");
      extendedKeys.put(3085, "CTRL-ALT-ENTER");
      extendedKeys.put(3581, "CTRL-ALT-SHIFT-TAB");
      extendedKeys.put(3584, "CTRL-ALT-DELETE");
      extendedKeys.put(3597, "CTRL-SHIFT-ALT-ENTER");
      extendedKeys.put(4093, "CTRL-SHIFT-ALT-SHIFT-TAB");

      // filling the map of alternate key labels
      addAltLabel("RETURN", 13);
      addAltLabel("BACK-TAB", 509);

      addAltLabel(MOUSE_MOVE_DOWN,      keyCode(MOUSE_SELECT_DOWN));
      addAltLabel(MOUSE_MOVE_UP,        keyCode(MOUSE_SELECT_UP));
      addAltLabel(MOUSE_MOVE_CLICK,     keyCode(MOUSE_SELECT_CLICK));
      addAltLabel(MOUSE_MOVE_DBLCLICK,  keyCode(MOUSE_SELECT_DBLCLICK));
   }
   
   /**
    * Get the mappings of standard functions to their associated keys.
    * 
    * @return   See above.
    */
   @Override
   protected Map<String, String[]> standardKeyFunctions()
   {
      Map<String, String[]> res = new HashMap<>();
      
      res.put(APPEND_LINE, new String[] {"CTRL-A"});
      
      res.put(BACK_TAB,      new String[] {"SHIFT-TAB", "CTRL-U"});
      res.put(BACKSPACE,     new String[] {"BACKSPACE", "CTRL-H"});
      res.put(BLOCK,         new String[] {"CTRL-V"});
      res.put(BOTTOM_COLUMN, new String[] {"CTRL-ALT-F1"});
      res.put(BREAK_LINE,    new String[] {"CTRL-ALT-B"});

      res.put(CANCEL_PICK,  new String[] {"CTRL-ALT-F2"});
      res.put(CHOICES,      new String[] {"CTRL-ALT-F3"});
      res.put(CLEAR,        new String[] {"F8", "CTRL-Z"});
      res.put(CLOSE,        new String[] {"CTRL-ALT-Z"});
      res.put(COMPILE,      new String[] {"CLTR-ALT-P"});
      res.put(COPY,         new String[] {"F11", "CTRL-ALT-C", "ALT-F1"});
      res.put(CURSOR_DOWN,  new String[] {"CURSOR-DOWN", "CTRL-J"});
      res.put(CURSOR_LEFT,  new String[] {"CURSOR-LEFT", "CTRL-O"});
      res.put(CURSOR_RIGHT, new String[] {"CURSOR-RIGHT", "CTRL-L"});
      res.put(CURSOR_UP,    new String[] {"CURSOR-UP", "CTRL-K"});
      res.put(CUT,          new String[] {"F10", "CTRL-ALT-X"});

      res.put(DEFAULT_POP_UP,   new String[] {"CTRL-ALT-U", "SHIFT-F4"});
      res.put(DELETE_CHARACTER, new String[] {"DEL", "DELETE"});
      res.put(DELETE_COLUMN,    new String[] {"CTRL-ALT-F4"});
      res.put(DELETE_END_LINE,  new String[] {"CTRL-ALT-K"});
      res.put(DELETE_FIELD,     new String[] {"CTRL-ALT-F5"});
      res.put(DELETE_LINE,      new String[] {"CTRL-D"});
      res.put(DELETE_WORD,      new String[] {"CTRL-ALT-D"});

      res.put(EDITOR_BACKTAB, new String[] {"CTRL-B"});
      res.put(EDITOR_TAB,     new String[] {"CTRL-G", "TAB"});
      res.put(END,            new String[] {"END"});
      res.put(END_ERROR,      new String[] {"F4", "ESC", "CTRL-E"});
      res.put(ENTER_MENUBAR,  new String[] {"F3", "ALT"});
      res.put(ESC_FUNC,       new String[] {"ESC"});
      res.put(EXIT,           new String[] {"CTRL-ALT-Q", "EXIT"});

      res.put(FIND,          new String[] {"CTRL-F", "FIND"});
      res.put(FIND_NEXT,     new String[] {"CTRL-ALT-F"});
      res.put(FIND_PREVIOUS, new String[] {"CTRL-ALT-I"});

      res.put(GET,  new String[] {"F5", "CTRL-ALT-O"});  
      res.put(GO,   new String[] {"F1", "CTRL-X"});
      res.put(GOTO, new String[] {"CTRL-ALT-G"});

      res.put(HELP,  new String[] {"CTRL-ALT-H", "F2"});
      res.put(HOME,  new String[] {"HOME"});

      res.put(INSERT_COLUMN,      new String[] {"CTRL-ALT-F6"});
      res.put(INSERT_FIELD,       new String[] {"CTRL-ALT-F7"});
      res.put(INSERT_FIELD_DATA,  new String[] {"CTRL-ALT-F8"});
      res.put(INSERT_FIELD_LABEL, new String[] {"CTRL-ALT-F9"});
      res.put(INSERT_MODE,        new String[] {"F9", "CTRL-T", "INS"});

      res.put(LEFT_END, new String[] {"ALT-CURSOR-LEFT"});

      res.put(MAIN_MENU, new String[] {"CTRL-ALT-F10"});
      res.put(MOVE,      new String[] {"CTRL-ALT-F11"});

      res.put(NEW,        new String[] {"CTRL-ALT-N"});
      res.put(NEW_LINE,   new String[] {"CTRL-N"});
      res.put(NEXT_ERROR, new String[] {"CTRL-ALT-E"});
      res.put(NEXT_FRAME, new String[] {"CTRL-TAB"});
      res.put(NEXT_WORD,  new String[] {"CTRL-CURSOR-RIGHT", "CTRL-W"});

      res.put(OPEN_LINE_ABOVE, new String[] {"CTRL-ALT-L"});
      res.put(OPTIONS,         new String[] {"CTRL-ALT-F12"});

      res.put(PAGE_DOWN,       new String[] {"PAGE-DOWN"});
      res.put(PAGE_LEFT,       new String[] {"CTRL-ALT-W"});
      res.put(PAGE_RIGHT,      new String[] {"CTRL-ALT-Y"});
      // res.put(PAGE_RIGHT_TEXT, new String[] {"CTRL-ALT-J"}); TODO: found in key report
      res.put(PAGE_UP,         new String[] {"PAGE-UP"});
      res.put(PASTE,           new String[] {"F12", "CTRL-ALT-V", "ALT-F2"});
      res.put(PICK,            new String[] {"CTRL-SHIFT-F1"});
      res.put(PICK_AREA,       new String[] {"CTRL-SHIFT-F2"});
      res.put(PICK_BOTH,       new String[] {"CTRL-SHIFT-F3"});
      res.put(PREV_FRAME,      new String[] {"CTRL-SHIFT-TAB"});
      res.put(PREV_WORD,       new String[] {"CTRL-CURSOR-LEFT", "CTRL-P"});
      res.put(PUT,             new String[] {"F6"});

      res.put(RECALL,         new String[] {"CTRL-R", "F7"});
      res.put(REPLACE,        new String[] {"CTRL-ALT-R"});
      res.put(REPORTS,        new String[] {"CTRL-SHIFT-F4"});
      res.put(RESUME_DISPLAY, new String[] {"CTRL-Q"});
      res.put(RETURN,         new String[] {"CTRL-M", "ENTER"});
      res.put(RIGHT_END,      new String[] {"ALT-CURSOR-RIGHT"});

      res.put(SAVE_AS,      new String[] {"CTRL-ALT-A"});
      res.put(SCROLL_LEFT,  new String[] {"CTRL-SHIFT-F5"});
      res.put(SCROLL_MODE,  new String[] {"CTRL-ALT-T"});
      res.put(SCROLL_RIGHT, new String[] {"CTRL-SHIFT-F6"});
      res.put(SETTINGS,     new String[] {"CTRL-SHIFT-F7"});
      res.put(STOP,         new String[] {"CTRL-BREAK"});
      res.put(STOP_DISPLAY, new String[] {"CTRL-S"});

      res.put(TAB,        new String[] {"TAB", "CTRL-I"});
      res.put(TOP_COLUMN, new String[] {"CTRL-SHIFT-F8"});
      
      // special entries for key actions
      res.put(ENDKEY, new String[] {});
      res.put(ERROR,  new String[] {});
      
      return res;
   }
   
   /**
    * Check if the given key code is treated by rule 1.
    * 
    * @param    code
    *           The key code to check.
    *           
    * @return   <code>true</code> if rule 1 must be applied to the key code.
    */
   @Override
   protected boolean useRule1(int code)
   {
      return (code >= 1013 && code <= 1023) ||
             (code >= 1025 && code <= 1279) ||
             super.useRule1(code);
   }
   
   /**
    * Check if the given key code is treated by rule 3.
    * 
    * @param    code
    *           The key code to check.
    *           
    * @return   <code>true</code> if rule 3 must be applied to the key code.
    */
   @Override
   protected boolean useRule3(int code)
   {
      return code == 634 || super.useRule3(code);
   }
}