/*
** Module   : VT100ProgressKeyboard.java
** Abstract : VT-100 keyboard implementation that has 4GL-specific key remapping
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.

**
** -#- -I- --Date-- --------------------------------Description---------------------------------
** 001 GES 20150414 First version with core key processing moved from Swing-specific code.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.chui.driver;

// this usage of AWT events is intentional, they are being used separately from the
// AWT/Swing event model itself; no other AWT/Swing usage is occurring in this class
import java.awt.event.KeyEvent;

import com.goldencode.terminal.*;
import com.goldencode.util.*;

/**
 * VT-100 keyboard implementation with 4GL-specific key remapping.
 */
public class VT100ProgressKeyboard
extends VT100AbstractKeyboard
{  
   /** RETURN key code */
   public static final char RETURN = '\r';

   /** OS platform flag */
   private boolean isWin = false;
   
   /**
    * Constructor.
    */
   public VT100ProgressKeyboard()
   {
      isWin = PlatformHelper.isUnderWindowsFamily();
   }
   
   /**
    * Handle key strokes. VT100AbstractKeyboard class has been designed to emulate a VT100
    * keyboard and some keystrokes are transformed before sending to terminal.
    * <p>
    * Console applications on Windows OS do not use VT100 keyboard codes. In this case we
    * only send the character code, we don't send the key code (the key code is forced to -1).
    * 
    * @param    keyCode
    *           The key code from the input event.
    * @param    charCode
    *           The character code from the input event.
    */
   public void handle(int keyCode, char charCode)
   {
      if (isWin)
      {
         switch (keyCode)
         {
            case KeyEvent.VK_CONTROL:
            case KeyEvent.VK_SHIFT:
            case KeyEvent.VK_ALT:
            case KeyEvent.VK_CAPS_LOCK:    
            case KeyEvent.VK_SCROLL_LOCK:
            case KeyEvent.VK_NUM_LOCK:
            case KeyEvent.VK_PAUSE:
            case KeyEvent.VK_WINDOWS:
            case KeyEvent.VK_CONTEXT_MENU:
               break;
               
            case KeyEvent.VK_BACK_SPACE:
               handleKeystroke(-1, (char) 0x08);
               break;
               
            default:
               handleKeystroke(-1, charCode);
               
               // if a CR was input, we must add the LF
               if (charCode == RETURN)
               {
                  handleKeystroke(-1, '\n');
               }
               break;
         }
      }
      else
      {
         handleKeystroke(keyCode, charCode);                  
      }
   }
   
   /**
    * Handle key inserts.
    * 
    * @param    keyCode
    *           Key code.
    * @param    charCode
    *           Character code
    */
   public void injectKey(int keyCode, char charCode)
   {
      handle(keyCode, charCode);
   }
}