/*
** Module   : WindowTitlePopupGuiImpl.java
** Abstract : GUI client implementation of window title system popup.
**
** Copyright (c) 2016-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -----------------------------Description-------------------------------------
** 001 VIG 20160216 Created initial version.
** 002 HC  20160607 GUI menus reworked to be displayed in dedicated top-level windows.
** 003 SBI 20160613 Changed to register maximize, minimize and close system menu actions.
** 004 HC  20170823 Removed unused code of the window-state runtime logic. Added runtime support
**                  of WINDOW:SHOW-IN-TASKBAR.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.gui;

import java.awt.event.*;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.event.MouseEvt;
import com.goldencode.p2j.ui.client.gui.driver.MouseHandler;
import com.goldencode.p2j.ui.client.gui.driver.SystemAction;

/**
 * GUI client implementation class of "window title" system popup. 
 */
public class WindowTitlePopupGuiImpl 
extends MenuGuiImpl
{
   /** The restore menu option.*/
   private MenuItemGuiImpl restoreOption;

   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param    owner
    *           A widget that will own the popup.
    */
   private WindowTitlePopupGuiImpl(TopLevelWindow<?> owner)
   {
      super(null, new MenuConfig());
      config().popupOnly = true;
      config().ownerId = owner.getId().asInt();
   }
   
   /**
    * Create a system popup menu.
    * 
    * @param   owner
    *          The window owning created system menu.
    *   
    * @return   A popup widget.
    */
   static WindowTitlePopupGuiImpl create(TopLevelWindow<?> owner)
   {
      boolean dlgBox = owner instanceof ModalWindow;
      
      WindowTitlePopupGuiImpl menu = new WindowTitlePopupGuiImpl(owner);
      menu.initialize(WidgetId.nextID(), menu.config());
      
      final WindowGuiImpl window = (WindowGuiImpl) owner.ancestor();
      
      MenuItemGuiImpl moveOption = MenuItemGuiImpl.createDynamicItem("Move", false, new MouseAdapter()
      {
         @Override
         public void mouseClicked(MouseEvent e)
         {
            // TODO: implement this
            // WindowGuiImpl.this.move();
         }
      });
      
      MenuItemGuiImpl closeOption = MenuItemGuiImpl.createDynamicItem("Close", true, new MouseAdapter()
      {
         @Override
         public void mouseClicked(MouseEvent e)
         {
            window.close();
         }
      });
      
      if (!dlgBox)
      {
         menu.restoreOption = 
                  MenuItemGuiImpl.createDynamicItem("Restore", false, new MouseAdapter()
         {
            @Override
            public void mouseClicked(MouseEvent e)
            {
               window.restore(true);
               menu.restoreOption.setEnabled(false);
            }
         });

         menu.add(menu.restoreOption);
      }
      
      menu.add(moveOption);
      if (!dlgBox)
      {
         MenuItemGuiImpl sizeOption = 
                  MenuItemGuiImpl.createDynamicItem("Size", false, new MouseAdapter()
         {
            @Override
            public void mouseClicked(MouseEvent e)
            {
               // TODO: implement this
               // WindowGuiImpl.this.size();
            }
         });

         menu.add(sizeOption);
         
         WindowConfig cfg = window.config();
         if (cfg.controlBox)
         {
            MenuItemGuiImpl minimizeOption =
            MenuItemGuiImpl.createDynamicItem("Minimize", true, new MouseAdapter()
            {
               @Override
               public void mouseClicked(MouseEvent e)
               {
                  window.iconify(true);
               }
            });

            menu.add(minimizeOption);
            // to handle the system menu minimize action by the web client
            int menuId = minimizeOption.getId().asInt();
            MouseHandler.registerWidgetAction(
            menuId, new SystemAction(menuId, MouseEvt.MINIMIZE, null));

            MenuItemGuiImpl maximizeOption =
            MenuItemGuiImpl.createDynamicItem("Maximize", true, new MouseAdapter()
            {
               @Override
               public void mouseClicked(MouseEvent e)
               {
                  window.maximize(true);
                  menu.restoreOption.setEnabled(true);
               }
            });

            menu.add(maximizeOption);
         }
      }
      
      menu.add(closeOption);
      
      return menu;
   }
}