/*
** Module   : CursorType.java
** Abstract : defines well known cursor types
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.

**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 GES 20150501 First version.
** 002 IAS 20151214 Added Swing cursor type and new pre-defined types.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.gui.driver;

//this usage of AWT Cursor is intentional, it is being used separately from the
//rest of AWT/Swing; no other AWT/Swing usage is occurring in this class
import java.awt.Cursor;
import java.util.*;

/**
 * This is the list of all possible cursor types.
 */
public enum CursorType
{
   DEFAULT(Cursor.DEFAULT_CURSOR),
   TEXT(Cursor.TEXT_CURSOR),
   N_RESIZE(Cursor.N_RESIZE_CURSOR),
   NE_RESIZE(Cursor.NE_RESIZE_CURSOR),
   E_RESIZE(Cursor.E_RESIZE_CURSOR),
   SE_RESIZE(Cursor.SE_RESIZE_CURSOR),
   S_RESIZE(Cursor.S_RESIZE_CURSOR),
   SW_RESIZE(Cursor.SW_RESIZE_CURSOR),
   W_RESIZE(Cursor.W_RESIZE_CURSOR),
   NW_RESIZE(Cursor.NW_RESIZE_CURSOR),
   CROSS(Cursor.CROSSHAIR_CURSOR),
   WAIT(Cursor.WAIT_CURSOR),
   MOVE(Cursor.MOVE_CURSOR),
   HAND(Cursor.HAND_CURSOR),
   CUSTOM(Cursor.CUSTOM_CURSOR)
   ;
   
   /* AWT custor type */
   public final int awtCursor;

   /**
    * Constructor
    * 
    * @param awtCursor
    *        AWT custor type
    */
   private CursorType(int awtCursor)
   {
      this.awtCursor = awtCursor;
   }
   
   /**
    * Retrieve CursorType by AWT type
    * 
    * @param awtCursor
    *        AWT cursor type
    *        
    * @return corresponding CursorType
    */
   public static CursorType fromAwtCursor(int awtCursor)
   {
     return  Arrays.stream(values()).filter(c -> c.awtCursor == awtCursor).
                                                        findFirst().orElse(DEFAULT);
   }
}