/*
** Module   : GuiWindowContainer.java
** Abstract : GUI window container.
**
** Copyright (c) 2015-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 HC  20160612 Initial version.
** 002 HC  20150702 Implemented window modality independently on the actual UI driver.
** 003 HC  20151013 Made the container focusable.
** 004 HC  20151213 Improved DIALOG-BOX window parenting logic and related changes.
** 005 OM  20170320 Added support for Themes.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.gui.driver.swing;

import java.awt.*;
import java.lang.reflect.InvocationTargetException;

import javax.swing.*;

import com.goldencode.p2j.ui.client.driver.swing.SwingKeyboardReader;

/**
 * The class implements JDialog-based window container for
 * Swing application windows. 
 */
@SuppressWarnings("serial")
public class GuiWindowContainer
extends JWindow
{
   /** Simulator instance. */
   private SwingEmulatedWindow sim = null;

   /**
    * Construct the new window container by instantiating the {@link SwingEmulatedWindow} 
    * as the holder of the content pane.
    * 
    * @param    owner
    *           The owning window.
    * @param    windowTitle
    *           Window container title.
    * @param    keyReader
    *           The {@link SwingKeyboardReader key reader} which listens for incoming keys.
    */
   private GuiWindowContainer(Window owner,
                              String windowTitle,
                              SwingKeyboardReader keyReader)
   {
      super(owner);
      setFocusable(true);
      
      // common initialization
      sim = new SwingEmulatedWindow(this, keyReader);
      init(sim.getContentPane());
   }

   /**
    * Initialize the window container by creating a new instance of this class.
    * The created emulated window will receive an owner window from the call of
    * {@link SwingEmulatedWindow#getContainingWindow()} of the <code>owner</code>
    * parameter.
    * 
    * @param    owner
    *           Owning window. May be null if the initializing window container is not
    *           expected to have an owning window.
    * @param    windowTitle
    *           Window container title.
    * @param    keyReader
    *           The {@link SwingKeyboardReader key reader} which listens for incoming keys.
    *           
    * @return   The simulator instance.
    */
   public static SwingEmulatedWindow load(final SwingEmulatedWindow owner,
                                          final String windowTitle,
                                          final SwingKeyboardReader keyReader) 
   throws InterruptedException,
          InvocationTargetException
   {
      // lame hack because of Java flaw
      final GuiWindowContainer[] container = new GuiWindowContainer[1];

      // instantiate the application's main frame
      SwingUtilities.invokeAndWait(() ->
      {
         Window owningWindow = owner.getContainingWindow();
         container[0] = new GuiWindowContainer(owningWindow, windowTitle, keyReader);
      });

      return container[0].sim;
   }
   
   /**
    * Initialize. GUI windows are not allowed to be closed via OS (i.e. ALT-F4).
    * 
    * @param   pane
    *          Content pane.
    */
   protected void init(JComponent pane)
   {
      setContentPane(pane);
   }
}