/*
** Module   : OuterFrame.java
** Abstract : Simple top-level window.
**
** Copyright (c) 2010-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SIY 20100629 Created initial version
** 002 SIY 20100804 Changes in imports, some refactorings.
** 003 SIY 20110112 Changes related to new color handling scheme.
** 004 CA  20140805 Added generics to force the abstract widget classes to not use implementation-
**                  specific code: the abstract widgets will have access only to the base APIs
**                  provided by the OutputManager.
**                  ChuiOutputManager is explicitly imported, to fine track the ChUI dependencies.
** 005 CA  20141119 Refactored drawing code to enable multiple OS windows, for GUI support.  Misc
**                  changes/fixes related to drawing on multiple windows.
** 006 EVL 20150424 Making the class GUI compatible.
** 007 EVL 20140613 Separating common drawing routine into protected method to avoid ChUI calls
**                  from GUI code.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.ui.client.widget;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.chui.driver.ChuiOutputManager;

/**
 * Simple top level frame.
 */
public class OuterFrame<O extends OutputManager<?>>
extends TitledWindow<O>
{
   /**
    * Constructor.
    * 
    * @param   title
    *          Frame title.
    */
   public OuterFrame(String title)
   {
      super(title, new Insets(1, 1, 1, 1));
   }

   /**
    * Constructor.
    * 
    * @param   insets
    *          Frame insets.
    */
   public OuterFrame(Insets insets)
   {
      super(null, insets);
   }
   
   /**
    * Draw widget.
    */
   @Override
   public void draw()
   {
      // get the current client mode
      boolean isChui = OutputManager.instance().isChui();
      
      if (isChui)
      {
         Color color = Color.DEFAULT;
         clear(location(), dimension(), color);

         if (insets().top != 0 && insets().bottom != 0)
         {
            // TODO: remove this dependency
            ((ChuiOutputManager) screen()).box(location(), dimension(), color);
         }

         if (hasTitle())
         {
            // TODO: remove this dependency
            ((ChuiOutputManager) screen()).at(location().offset(1, 0))
                                          .append(' ', color)
                                          .append(title(), color)
                                          .append(' ', color);
         }
      }

      drawInt();
   }

   /**
    * Internal drawing routine for common UI code.
    */
   protected void drawInt()
   {
      super.draw();
   }
}