/*
** Module   : CustomSecurityOps.java
** Abstract : Progress 4GL compatible security interface for _User security handlers.
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- ----------------------------------Description---------------------------------
** 001 OM  20130913 First version, declaring the basic four methods: get/set with and without the
**                  logical database name.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * The common interface for the classes that emulates USERID and SETUSERID functions of P4GL.
 */
public interface CustomSecurityOps
{
   /**
    * Returns the userid associated with the current connected database. If no database is
    * connected or there are at least two databases connected empty string is returned.
    *
    * @return   The current user context's userid or the empty string in the case of any problem.
    */
   public String getUserId();

   /**
    * Returns the userid associated with the current user of the given logical database.
    *
    * @param    dbname
    *           Logical database name.
    *
    * @return   The current user context's userid or the empty string in the
    *           case of any problem.
    */
   public String getUserIdFromDB(String dbname);

   /**
    * Authenticates the user for specified DB connection. Checks if the user login account match
    * the corresponding record of the _User table of the database. If there are multiple database
    * connections or none is connected, an error is issued.
    *
    * @param    userid
    *           The name of the user to set as UserID.
    * @param    password
    *           The users password.
    *
    * @return   The <code>true</code> in case of valid user match <code>false</code> otherwise.
    */
   public boolean setUserId(String userid, String password);

   /**
    * Authenticates the user for specified DB connection. Checks if the user login account match
    * the corresponding record of the _User table of the database.  If the specified database
    * is not connected, an error is issued.
    *
    *
    * @param    userid
    *           The name of the user to set as UserID.
    * @param    password
    *           The users password.
    * @param    dbname
    *           Logical database name.
    *
    * @return   The <code>true</code> in case of valid user match <code>false</code> otherwise.
    */
   public boolean setUserId(String userid, String password, String dbname);

   /**
    * Check the access level for a database. This will query the _User table and return the value.
    * 
    * @param   ldbname
    *          The logical database name to query. 
    * 
    * @return  The access level to specified database. 
    */
   public int getAuthLevel(String ldbname);

   /**
    * Checks if the currently authenticated user has access to a database.
    *
    * @param   ldbName
    *          The logical name of the database.
    *
    * @return  <code>true</code> if access to requested database is granted
    */
   public boolean hasAccessToDatabase(String ldbName);
}