/*
** Module   : LegacyLogManager.java
** Abstract : Define all APIs available via the LOG-MANAGER and DSLOG-MANAGER system handles.
**
** Copyright (c) 2014-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 GES 20141028 Created initial version.
** 002 EVL 20160224 Javadoc fixes to make compatible with Oracle Java 8 for Solaris 10.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * Exposes the following methods or attributes from the LOG-MANAGER and DSLOG-MANAGER system
 * handles:
 * <table summary="">
 * <tr>
 *    <th>Attribute or Method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>ENTRY-TYPES-LIST</td>
 *    <td>{@link #getEntryTypesList()}</td>
 * </tr>
 * <tr>
 *    <td>LOG-ENTRY-TYPES</td>
 *    <td>{@link #getLogEntryTypes()}, {@link #setLogEntryTypes(character)}</td>
 * </tr>
 * <tr>
 *    <td>LOGFILE-NAME</td>
 *    <td>{@link #getLogFileName()}, {@link #setLogFileName(character)}</td>
 * </tr>
 * <tr>
 *    <td>LOGGING-LEVEL</td>
 *    <td>{@link #getLoggingLevel()}, {@link #setLoggingLevel(integer)}</td>
 * </tr>
 * <tr>
 *    <td>LOG-THRESHOLD</td>
 *    <td>{@link #getLogThreshold()}</td>
 * </tr>
 * <tr>
 *    <td>NUM-LOG-FILES</td>
 *    <td>{@link #getNumLogFiles()}</td>
 * </tr>
 * <tr>
 *    <td>DB-CONTEXT</td>
 *    <td>{@link #getDBContext()}, {@link #setDBContext(character)}</td>
 * </tr>
 * <tr>
 *    <td>CLEAR-LOG()</td>
 *    <td>{@link #clearLog()}</td>
 * </tr>
 * <tr>
 *    <td>CLOSE-LOG()</td>
 *    <td>{@link #closeLog()}</td>
 * </tr>
 * <tr>
 *    <td>WRITE-MESSAGE()</td>
 *    <td>{@link #writeMessage(character)}, {@link #writeMessage(character, character)}</td>
 * </tr>
 * </table>
 * This interface is used as a returned type by the {@link handle#unwrapLogManager}
 * API, which, depending on the handle's state, it either returns an 
 * {@link LegacyLogManager} instance or a dynamic proxy using
 * {@link handle#invalidAttrAccessProxy}. 
 */
@LegacyResource(resource = LegacyResource.PSEUDO_WIDGET)
public interface LegacyLogManager
extends CommonHandle,
        InstantiatingProcedure
{
   /**
    * Provides the comma-separated list of currently valid entry types that can be assigned to
    * the {@link #setLogEntryTypes}.
    *
    * @return   The list of valid log entry types.
    */
   @LegacyAttribute(name = "ENTRY-TYPES-LIST")
   public character getEntryTypesList();
   
   /**
    * Provides the comma-separated list of entry types that are being actively used for logging.
    *
    * @return   The list of active log entry types.
    */
   @LegacyAttribute(name = "LOG-ENTRY-TYPES")
   public character getLogEntryTypes();
   
   /**
    * Modifies the list of log entry types that are being actively used for logging.
    *
    * @param    types
    *           The comma-separated list of active entry types.
    */
   @LegacyAttribute(name = "LOG-ENTRY-TYPES", setter = true)
   public void setLogEntryTypes(String types);
   
   /**
    * Modifies the list of log entry types that are being actively used for logging.
    *
    * @param    types
    *           The comma-separated list of active entry types.
    */
   @LegacyAttribute(name = "LOG-ENTRY-TYPES", setter = true)
   public void setLogEntryTypes(character types);
   
   /**
    * Obtains the filename to which log entries and stack traces are written.
    *
    * @return   The filename.
    */
   @LegacyAttribute(name = "LOGFILE-NAME")
   public character getLogFileName();
   
   /**
    * Assigns the filename to which log entries and stack traces are written.
    *
    * @param    name
    *           The log file name.
    */
   @LegacyAttribute(name = "LOGFILE-NAME", setter = true)
   public void setLogFileName(String name);
   
   /**
    * Assigns the filename to which log entries and stack traces are written.
    *
    * @param    name
    *           The log file name.
    */
   @LegacyAttribute(name = "LOGFILE-NAME", setter = true)
   public void setLogFileName(character name);
   
   /**
    * Obtains the default logging level for all logging categories.
    *
    * @return   The logging level.
    */
   @LegacyAttribute(name = "LOGGING-LEVEL")
   public integer getLoggingLevel();
   
   /**
    * Assigns the default logging level for all logging categories.
    * <p>
    * There are 5 valid levels:
    * <p>
    * <ul>
    *   <li> 0 - no logging
    *   <li> 1 - only errors
    *   <li> 2 - basic, each entry type determines its own output but all types will generate
    *            some output
    *   <li> 3 - verbose
    *   <li> 4 - extended
    * </ul>
    *
    * @param    level
    *           The logging level to use by default.
    */
   @LegacyAttribute(name = "LOGGING-LEVEL", setter = true)
   public void setLoggingLevel(int level);
   
   /**
    * Assigns the default logging level for all logging categories.
    * <p>
    * There are 5 valid levels:
    * <p>
    * <ul>
    *   <li> 0 - no logging
    *   <li> 1 - only errors
    *   <li> 2 - basic, each entry type determines its own output but all types will generate
    *            some output
    *   <li> 3 - verbose
    *   <li> 4 - extended
    * </ul>
    *
    * @param    level
    *           The logging level to use by default.
    */
   @LegacyAttribute(name = "LOGGING-LEVEL", setter = true)
   public void setLoggingLevel(integer level);
   
   /**
    * Obtains the configured log file size in bytes at which the log file will be renamed and a
    * new log file started.
    *
    * @return   The maximum log file size.
    */
   @LegacyAttribute(name = "LOG-THRESHOLD")
   public integer getLogThreshold();
   
   /**
    * Obtains the configured number of log files that will be stored in total (including both
    * backed up log files and the current log).
    *
    * @return   The maximum number of log files.
    */
   @LegacyAttribute(name = "NUM-LOG-FILES")
   public integer getNumLogFiles();
   
   /**
    * Obtains the logical database name of database that the DSLOG-MANAGER is generating log
    * entries.
    *
    * @return   The logical database name or <code>unknown</code> if no database name has been
    *           assigned.
    */
   @LegacyAttribute(name = "DB-CONTEXT")
   public character getDBContext();
   
   /**
    * Assigns the logical database name of database that the DSLOG-MANAGER is generating log
    * entries.
    *
    * @param    ldbname
    *           The logical database name.
    */
   @LegacyAttribute(name = "DB-CONTEXT", setter = true)
   public void setDBContext(String ldbname);
   
   /**
    * Assigns the logical database name of database that the DSLOG-MANAGER is generating log
    * entries.
    *
    * @param    ldbname
    *           The logical database name.
    */
   @LegacyAttribute(name = "DB-CONTEXT", setter = true)
   public void setDBContext(character ldbname);
   
   /**
    * Delete all messages from the currently open log file.
    *  
    * @return   <code>true</code> if the operation succeeded, or <code>false</code>
    *           otherwise.
    */
   @LegacyMethod(name = "CLEAR-LOG")
   public logical clearLog();
   
   /**
    * Close the currently open log file.  Additional logging will not be possible after this
    * operation.
    *  
    * @return   <code>true</code> if the operation succeeded, or <code>false</code>
    *           otherwise.
    */
   @LegacyMethod(name = "CLOSE-LOG")
   public logical closeLog();
   
   /**
    * Write a user defined message to the currently open log file using the default subsystem
    * name "APPL".
    *
    * @param    msg
    *           The message to write.
    *  
    * @return   <code>true</code> if the operation succeeded, or <code>false</code>
    *           otherwise.
    */
   @LegacyMethod(name = "WRITE-MESSAGE")
   public logical writeMessage(String msg);
   
   /**
    * Write a user defined message to the currently open log file using the default subsystem
    * name "APPL".
    *
    * @param    msg
    *           The message to write.
    *  
    * @return   <code>true</code> if the operation succeeded, or <code>false</code>
    *           otherwise.
    */
   @LegacyMethod(name = "WRITE-MESSAGE")
   public logical writeMessage(character msg);
   
   /**
    * Write a user defined message to the currently open log file using the given subsystem
    * name.
    *
    * @param    msg
    *           The message to write.
    * @param    subsys
    *           Subsystem name (up to 10 characters will be used).
    *  
    * @return   <code>true</code> if the operation succeeded, or <code>false</code>
    *           otherwise.
    */
   @LegacyMethod(name = "WRITE-MESSAGE")
   public logical writeMessage(String msg, String subsys);
   
   /**
    * Write a user defined message to the currently open log file using the given subsystem
    * name.
    *
    * @param    msg
    *           The message to write.
    * @param    subsys
    *           Subsystem name (up to 10 characters will be used).
    *  
    * @return   <code>true</code> if the operation succeeded, or <code>false</code>
    *           otherwise.
    */
   @LegacyMethod(name = "WRITE-MESSAGE")
   public logical writeMessage(character msg, String subsys);
   
   /**
    * Write a user defined message to the currently open log file using the given subsystem
    * name.
    *
    * @param    msg
    *           The message to write.
    * @param    subsys
    *           Subsystem name (up to 10 characters will be used).
    *  
    * @return   <code>true</code> if the operation succeeded, or <code>false</code>
    *           otherwise.
    */
   @LegacyMethod(name = "WRITE-MESSAGE")
   public logical writeMessage(String msg, character subsys);
   
   /**
    * Write a user defined message to the currently open log file using the given subsystem
    * name.
    *
    * @param    msg
    *           The message to write.
    * @param    subsys
    *           Subsystem name (up to 10 characters will be used).
    *  
    * @return   <code>true</code> if the operation succeeded, or <code>false</code>
    *           otherwise.
    */
   @LegacyMethod(name = "WRITE-MESSAGE")
   public logical writeMessage(character msg, character subsys);
}