/*
** Module   : Profiler.java
** Abstract : Define all APIs available via the PROFILER system handle.
**
** Copyright (c) 2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20180510 Created initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * Exposes the following methods or attributes from the PROFILER handle:
 * <table summary="">
 * <tr>
 *    <th>PROFILER attribute or method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>PROFILING</td>
 *    <td>{@link #isProfiling()} and {@link #setProfiling(logical)}</td>
 * </tr>
 * <tr>
 *    <td>ENABLED</td>
 *    <td>{@link #isEnabled()} and {@link #setEnabled(logical)}</td>
 * </tr>
 * <tr>
 *    <td>DESCRIPTION</td>
 *    <td>{@link #getDescription()} and {@link #setDescription(character)}</td>
 * </tr>
 * <tr>
 *    <td>LISTINGS</td>
 *    <td>{@link #isListings()} and {@link #setListings(logical)}</td>
 * </tr>
 * </table>
 * This interface is used as a returned type by the 
 * {@link com.goldencode.p2j.util.handle#unwrapProfiler()} API, which, depending on the
 * handle's state, it either returns an {@link ProfilerUtils#asHandle()}
 * instance or a dynamic proxy using {@link handle#invalidAttrAccessProxy}.
 */
@LegacyResource(resource = LegacyResource.PSEUDO_WIDGET)
public interface Profiler
extends CommonHandle,
        CommonFile,
        Enabled
{
   /**
    * Get a the instance for the PROFILER system handle.
    * 
    * @return   The handle to the object.
    */
   @LegacyAttribute(name = "HANDLE")
   public handle asHandle();

   /**
    * Set the value of PROFILER:PROFILING attribute. 
    * 
    * @param   profiling 
    *          The new value of PROFILER:PROFILING attribute.
    */
   @LegacyAttribute(name = "PROFILING", setter = true)
   public void setProfiling(logical profiling);

   /**
    * Get the value of PROFILER:PROFILING attribute. 
    * 
    * @return  the value of PROFILER:PROFILING attribute.
    */
   @LegacyAttribute(name = "PROFILING")
   public logical isProfiling();

   /**
    * Set the value of PROFILER:DESCRIPTION attribute. 
    * 
    * @param   description 
    *          The new value of PROFILER:DESCRIPTION attribute.
    */
   @LegacyAttribute(name = "DESCRIPTION", setter = true)
   public void setDescription(character description);

   /**
    * Get the value of PROFILER:DESCRIPTION attribute. 
    * 
    * @return  the value of PROFILER:DESCRIPTION attribute.
    */
   @LegacyAttribute(name = "DESCRIPTION")
   public character getDescription();

   /**
    * Set the value of PROFILER:LISTINGS attribute. 
    * 
    * @param   listings 
    *          The new value of PROFILER:LISTINGS attribute.
    */
   @LegacyAttribute(name = "LISTINGS", setter = true)
   public void setListings(logical listings);

   /**
    * Get the value of PROFILER:LISTINGS attribute. 
    * 
    * @return  the value of PROFILER:LISTINGS attribute.
    */
   @LegacyAttribute(name = "LISTINGS")
   public logical isListings();
}