/*
** Module   : SOAPHeader.java
** Abstract : Interface defining attributes/methods for SOAP-HEADER resource.
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20130216 Created initial version.
** 002 CA  20130221 Added ADM-DATA and UNIQUE-ID.
** 003 CA  20130927 Added LegacyResource annotation, to determine resource type.
** 004 EVL 20131008 Adding legacy attributes and methods annotation.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * Interface describing the features for SOAP-HEADER resource:
 * <ul>
 *    <li>{@link #getNumHeaderEntries NUM-HEADER-ENTRIES attribute}</li>
 *    <li>{@link #addHeaderEntry ADD-HEADER-ENTRY method}</li>
 *    <li>{@link #getHeaderEntry GET-HEADER-ENTRY method}</li>
 * </ul>
 */
@LegacyResource(resource = LegacyResource.SOAP_HEADER)
public interface SOAPHeader
extends ADMData,
        UniqueID
{
   /**
    * Returns the number of SOAP-HEADER-ENTRYREF object entries attached to the SOAP-HEADER 
    * object.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "NUM-HEADER-ENTRIES")
   public integer getNumHeaderEntries();

   /**
    * Creates a new entry in a SOAP-header object's list of entries and associates the new entry
    * with the given SOAP-HEADER-ENTRY-REF object.
    * 
    * @param    entryRef
    *           The handle where the resource will be set.

    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "ADD-HEADER-ENTRY")
   public logical addHeaderEntry(handle entryRef);

   /**
    * Get the header entry at the given index and assign it to the given handle.
    * <p>
    * This method returns a logical even though the Progress 4GL documentation explicitly states 
    * otherwise. Actual working code has proven that the original documentation is incorrect. 
    * This implementation matches the actual runtime behavior of the 4GL.
    * 
    * @param    target
    *           The handle where the SOAP-HEADER-ENTRY instance will be saved.
    * @param    idx
    *           The index of the SOAP-HEADER-ENTRY instance to retrieve. 4GL allows only integer
    *           and int64 values to be passed to this parameter.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "GET-HEADER-ENTRY")
   public logical getHeaderEntry(handle target, long idx);

   /**
    * Get the header entry at the given index and assign it to the given handle.
    * <p>
    * This method returns a logical even though the Progress 4GL documentation explicitly states 
    * otherwise. Actual working code has proven that the original documentation is incorrect. 
    * This implementation matches the actual runtime behavior of the 4GL.
    * 
    * @param    target
    *           The handle where the SOAP-HEADER-ENTRY instance will be saved.
    * @param    idx
    *           The index of the SOAP-HEADER-ENTRY instance to retrieve. 4GL allows only integer
    *           and int64 values to be passed to this parameter.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "GET-HEADER-ENTRY")
   public logical getHeaderEntry(handle target, integer idx);

   /**
    * Get the header entry at the given index and assign it to the given handle.
    * <p>
    * This method returns a logical even though the Progress 4GL documentation explicitly states 
    * otherwise. Actual working code has proven that the original documentation is incorrect. 
    * This implementation matches the actual runtime behavior of the 4GL.
    * 
    * @param    target
    *           The handle where the SOAP-HEADER-ENTRY instance will be saved.
    * @param    idx
    *           The index of the SOAP-HEADER-ENTRY instance to retrieve. 4GL allows only integer
    *           and int64 values to be passed to this parameter.
    * 
    * @return   <code>true</code> if the operation was possible.
    */
   @LegacyMethod(name = "GET-HEADER-ENTRY")
   public logical getHeaderEntry(handle target, int64 idx);
}