/*
** Module   : SocketListener.java
** Abstract : An interface holding the SERVER-SOCKET specific methods and 
**            attributes api.
**
** Copyright (c) 2013-2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 CS  20130204 Created initial version.
** 002 CA  20130908 Javadoc changes.
** 003 CA  20130927 Added LegacyResource annotation, to determine resource type.
** 004 EVL 20131008 Adding legacy attributes and methods annotation.
** 005 GES 20141124 Javadoc fix.
** 006 EVL 20160224 Javadoc fixes to make compatible with Oracle Java 8 for Solaris 10.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

/**
 * Interface for SERVER-SOCKET methods and attributes
 * <p>
 * Exposes the following Attributes and methods:
 * <table summary="">
 * <tr>
 *    <th>attribute or method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>ENABLE-CONNECTIONS</td>
 *    <td>{@link #enableConnections(String)}, 
 *        {@link #enableConnections(character)}
 *    </td>
 * </tr>
 * <tr>
 *    <td>DISABLE-CONNECTIONS</td>
 *    <td>{@link #disableConnections()}</td>
 * </tr>
 * <tr>
 *    <td>SET-CONNECT-PROCEDURE</td>
 *    <td>{@link #setConnectProcedure(String)},
 *        {@link #setConnectProcedure(character)},
 *        {@link #setConnectProcedure(String, handle) },
 *        {@link #setConnectProcedure(character, handle)},
 *        
 *    </td>
 * </tr>
 * </table>
 * This interface is used as a returned type by the 
 * {@link handle#unwrapSocketListener} API, which, will return a
 * {@link SocketListenerImpl} for a Server-Socket type handle.
 */
@LegacyResource(resource = LegacyResource.SERVER_SOCKET)
public interface SocketListener 
extends CommonHandleChain
{
   /**
    * Java implementation for the ENABLE-CONNECTION() method.  This will attempt to start 
    * listening on the specified port and will return <code>true</code> if operation succeeded, 
    * or <code>false</code> otherwise.
    * <p>
    * The following options can be specified as valid ENABLE-CONNECTIONS() options:
    * <ul>
    *    <li><b>-S port</b> The port to which to connect.</li>
    *    <li><b>-ssl</b> If specified the connection will in ssl mode.</li>
    *    <li><b>-pf config-file-name</b> Specifies the file from which options will be read.</li>
    *    <li><b>-qsize maxconn</b> The maximum number of connections to handle  before refusing 
    *        extra incoming connections.</li>
    *    <li><b>-keyalias alias</b> The alias for the digital certificate, default is:
    *        <b>default_server.</b></li>
    *    <li><b>-keyaliaspasswd pwd</b> The password for the key of the digital certificate. 
    *        To use always with -keyalias option.</li>
    *    <li><b>-nossesioncache</b> Disables SSL client session cache.</li>
    *    <li><b>-sessiontimeout time</b> The number of seconds before the SSL client request is 
    *        timeout, default value is 180.</li>
    * </ul>
    * 
    * @param    option
    *           The set of options that define the enable-connections method.
    * 
    * @return   <code>true</code> if operation succeeded, or <code>false</code> otherwise.
    */
   @LegacyMethod(name = "ENABLE-CONNECTIONS")
   public logical enableConnections(String option);
   
   /**
    * Java implementation for the ENABLE-CONNECTION() method.  This will attempt to start 
    * listening on the specified port and will return <code>true</code> if operation succeeded, 
    * or <code>false</code> otherwise.
    * <p>
    * The following options can be specified as valid ENABLE-CONNECTIONS() options:
    * <ul>
    *    <li><b>-S port</b> The port to which to connect.</li>
    *    <li><b>-ssl</b> If specified the connection will in ssl mode.</li>
    *    <li><b>-pf config-file-name</b> Specifies the file from which options will be read.</li>
    *    <li><b>-qsize maxconn</b> The maximum number of connections to handle  before refusing 
    *        extra incoming connections.</li>
    *    <li><b>-keyalias alias</b> The alias for the digital certificate, default is:
    *        <b>default_server.</b></li>
    *    <li><b>-keyaliaspasswd pwd</b> The password for the key of the digital certificate. 
    *        To use always with -keyalias option.</li>
    *    <li><b>-nossesioncache</b> Disables SSL client session cache.</li>
    *    <li><b>-sessiontimeout time</b> The number of seconds before the SSL client request is 
    *        timeout, default value is 180.</li>
    * </ul>
    * 
    * @param    option
    *           The set of options that define the enable-connections method.
    * 
    * @return   <code>true</code> if operation succeeded, or <code>false</code> otherwise.
    */
   @LegacyMethod(name = "ENABLE-CONNECTIONS")
   public logical enableConnections(character option);
   
   /**
    * Java implementation for the DISABLE-CONNECTIONS() SERVER-SOCKET method.  This will stop 
    * listening and accepting any new connections, but will interrupt the existing ones.
    * 
    * @return    <code>true</code> if the operation succeeded or <code>false</code> otherwise.
    */
   @LegacyMethod(name = "DISABLE-CONNECTIONS")
   public logical disableConnections();
   
   /**
    * Java implementation for the SET-CONNECT-PROCEDURE() SERVER-SOCKET method.  This will set 
    * the procedure to execute when a CONNECT event is received.
    * 
    * @param    procedureName
    *           The name of the procedure to be executed on CONNECT event.
    *         
    * @return   <code>true</code> if the operation was successful or <code>false</code> otherwise.
    */
   @LegacyMethod(name = "SET-CONNECT-PROCEDURE")
   public logical setConnectProcedure(String procedureName);
   
   /**
    * Java implementation for the SET-CONNECT-PROCEDURE() SERVER-SOCKET method.  This will set 
    * the procedure to execute when a CONNECT event is received.
    * 
    * @param    procedureName
    *           The name of the procedure to be executed on CONNECT event.
    *         
    * @return   <code>true</code> if the operation was successful or <code>false</code> otherwise.
    */
   @LegacyMethod(name = "SET-CONNECT-PROCEDURE")
   public logical setConnectProcedure(character procedureName);
   
   /**
    * Java implementation for the SET-CONNECT-PROCEDURE() SERVER-SOCKET method.  This will set 
    * the procedure to execute when a CONNECT event is received.
    * 
    * @param    procedureName
    *           The name of the procedure to be executed on CONNECT event.
    * @param    procHandle
    *           The context where the specified procedure belongs. Validation will be performed
    *           when the CONNECT event is executed.
    *         
    * @return   <code>true</code> if the operation was successful or <code>false</code> otherwise.
    */
   @LegacyMethod(name = "SET-CONNECT-PROCEDURE")
   public logical setConnectProcedure(String procedureName, handle procHandle);
   
   /**
    * Java implementation for the SET-CONNECT-PROCEDURE() SERVER-SOCKET method.  This will set 
    * the procedure to execute when a CONNECT event is received.
    * 
    * @param    procedureName
    *           The name of the procedure to be executed on CONNECT event.
    * @param    procHandle
    *           The context where the specified procedure belongs. Validation will be performed
    *           when the CONNECT event is executed.
    *         
    * @return   <code>true</code> if the operation was successful or <code>false</code> otherwise.
    */
   @LegacyMethod(name = "SET-CONNECT-PROCEDURE")
   public logical setConnectProcedure(character procedureName, handle procHandle);
}