/*
** Module   : TemplateHelper.java
** Abstract : Defines methods to fill templates with provided values.
**
** Copyright (c) 2018, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SBI 20180404 Initial version.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
**
** Additional terms under GNU Affero GPL version 3 section 7:
**
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
**
**   0. Attribution Requirement.
**
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
**
**   1. No License To Use Trademarks.
**
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
**
**   2. No Misrepresentation of Affiliation.
**
**     You may not represent yourself as Golden Code Development Corporation or FWD.
**
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
**
**   3. No Misrepresentation of Source or Origin.
**
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.Map;
import java.util.Set;
import java.util.function.Supplier;

/**
 * Defines helper static methods to fill templates with provided values.
 */
public class TemplateHelper
{
   /** The keys template */
   private static String KEY_TEMPLATE = "${%s}";
   
   /**
    * This private constructor is defined according to this class responsibility to provide helper
    * static methods to fill templates with values.
    */
   private TemplateHelper()
   {
   }
   
   /**
    * Process template and make any replacements of parameter values as needed. The replacement
    * placeholder follow the JEE expression language (EL) format for immediate evaluation. 
    * <p>
    * Example: ${expression}
    *
    * @param   template
    *          The template as an input stream.
    * @param   keysProvider
    *          The keys provider to fill gaps with values.
    * @param   writer
    *          The print writer to output the target document with help of the given template.
    */
   public static void fill(InputStream template,
                           Map<String, Supplier<String>> keysProvider,
                           PrintWriter writer)
   throws IOException
   {
      Set<String> keys = keysProvider.keySet();
      
      BufferedReader reader = new BufferedReader(new InputStreamReader(template));
      
      String str = null;
      
      // read the skeleton page and translate %substitutions
      // on the fly
      
      while (null != (str = reader.readLine()))
      {
         for(String key : keys)
         {
            String keyTemplate = String.format(KEY_TEMPLATE, key);
            if (str.contains(keyTemplate))
            {
               str = str.replace(keyTemplate, keysProvider.get(key).get());
            }
         }
         writer.println(str);
      }
      
      reader.close();
   }
}
