/*
** Module   : comauto_linux.c
** Abstract : Linux specific implementation for COM automation functions.
**
** Copyright (c) 2017, Golden Code Development Corporation.
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 EVL 20171024 Created initial version.
** 002 EVL 20171116 Changed signature of the property setter to include indexed properties.
**                  Adding extent based array support for property getter.
*/
/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

#include "comauto.h"

/**
 * This function will always return FALSE meaning no support for COM automation support in Linux.
 *
 * @return   FALSE in Linux OS.
 */
jboolean initComBridge()
{
   return JNI_FALSE;
}

/**
 * Gets the automation COM object. It can be a new instance or a reference to an already existing
 * top-level automation. 
 * 
 * @param    env
 *           Pointer to a structure that provides access to JNI environment.
 *
 * @param    name
 *           The name of the automation.
 *
 * @param    topLevel
 *           Only return top-level. If the requested COM name is not a top-level, return null.
 *
 * @param    noCreate
 *           Must exist. Do not allow creation of new instances. Only used with topLevel
 *           requests.
 *          
 * @return  The ID of the COM object, or 0 if such automation does not exist
 *          or it does not satisfy the parameter constraints. 
 */
jlong createCom(JNIEnv *env, jstring name, jboolean topLevel, jboolean noCreate)
{
   return 0; 
}

/**
 * Release and destroy the COM object with the specified ID.
 * 
 * @param    comPtrID
 *           The COM object ID.
 */
void releaseCom(jlong comPtrID)
{
   return;
}

/**
 * Get the specified COM object property.
 * 
 * @param    env
 *           Pointer to a structure that provides access to JNI environment.
 *
 * @param    comPtrID
 *           The COM object ID.
 *
 * @param    name
 *           The COM property name to retrieve.
 *           
 * @return   A BaseDataType instance with the COM property's details.
 */
jobject getComProperty(JNIEnv *env, jlong comPtrID, jstring name)
{
   return NULL;
}

/**
 * Set the specified COM object property.
 * 
 * @param    env
 *           Pointer to a structure that provides access to JNI environment.
 *
 * @param    comPtrID
 *           The COM object ID.
 *
 * @param    name
 *           The COM property name to set.
 *
 * @param    val
 *           The details about property's new value.
 *           
 * @param    ndxs
 *           Indices array values to find out indexed property.
 *           
 * @return   JNI_TRUE if the property was set successfully, JNI_FALSE otherwise.
 */
jboolean setComProperty(JNIEnv *env, jlong comPtrID, jstring name, jobject val, jobjectArray ndxs)
{
   return JNI_FALSE;
}

/**
 * Execute a COM object method call.
 * 
 * @param    env
 *           Pointer to a structure that provides access to JNI environment.
 *
 * @param    comPtrID
 *           The COM object ID.
 *
 * @param    name
 *           The COM method name to call.
 *
 * @param    valArray
 *           An array with this call's arguments supplied to the COM object method call.
 *           
 * @return   An 2d array where on index 0 are details of the returned value(single or array), and
 *           further indices correspond with argument's 1-based indices, where there will be a
 *           value(single or array) for each OUTPUT or INPUT-OUTPUT argument.
 */
jobjectArray callComMethod(JNIEnv *env, jlong comPtrID, jstring name, jobjectArray valArray)
{
   return NULL;
}

/**
 * Get the COM object name associated with the given COM object ID.
 * 
 * @param    env
 *           Pointer to a structure that provides access to JNI environment.
 *
 * @param    comPtrID
 *           The COM object ID.
 *           
 * @return   See above.
 */
jstring getComName(JNIEnv *env, jlong comPtrID)
{
   return NULL;
}

/**
 * Reports if this object is valid for use.  
 *
 * @param    comPtrID
 *           The COM object ID.
 *           
 * @return   JNI_TRUE if we are valid (can be used),
 *           JNI_FALSE otherwise(vas released or not created).
 */
jboolean isComValid(jlong comPtrID)
{
   return JNI_FALSE;
}
